// Generated by the protocol buffer compiler.  DO NOT EDIT!
// source: second/protos/preprocess.proto

#include "second/protos/preprocess.pb.h"

#include <algorithm>

#include <google/protobuf/io/coded_stream.h>
#include <google/protobuf/extension_set.h>
#include <google/protobuf/wire_format_lite.h>
#include <google/protobuf/descriptor.h>
#include <google/protobuf/generated_message_reflection.h>
#include <google/protobuf/reflection_ops.h>
#include <google/protobuf/wire_format.h>
// @@protoc_insertion_point(includes)
#include <google/protobuf/port_def.inc>

PROTOBUF_PRAGMA_INIT_SEG

namespace _pb = ::PROTOBUF_NAMESPACE_ID;
namespace _pbi = _pb::internal;

namespace second {
namespace protos {
PROTOBUF_CONSTEXPR Preprocess::Preprocess(
    ::_pbi::ConstantInitialized): _impl_{
    /*decltype(_impl_.random_global_rotation_min_rad_)*/0
  , /*decltype(_impl_.random_global_rotation_max_rad_)*/0
  , /*decltype(_impl_.random_global_scaling_min_)*/0
  , /*decltype(_impl_.random_global_scaling_max_)*/0
  , /*decltype(_impl_.random_noise_per_groundtruth_min_rad_)*/0
  , /*decltype(_impl_.random_noise_per_groundtruth_max_rad_)*/0
  , /*decltype(_impl_.random_noise_per_groundtruth_position_std_)*/0
  , /*decltype(_impl_._cached_size_)*/{}} {}
struct PreprocessDefaultTypeInternal {
  PROTOBUF_CONSTEXPR PreprocessDefaultTypeInternal()
      : _instance(::_pbi::ConstantInitialized{}) {}
  ~PreprocessDefaultTypeInternal() {}
  union {
    Preprocess _instance;
  };
};
PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PreprocessDefaultTypeInternal _Preprocess_default_instance_;
PROTOBUF_CONSTEXPR DatabasePreprocessingStep::DatabasePreprocessingStep(
    ::_pbi::ConstantInitialized): _impl_{
    /*decltype(_impl_.database_preprocessing_step_)*/{}
  , /*decltype(_impl_._cached_size_)*/{}
  , /*decltype(_impl_._oneof_case_)*/{}} {}
struct DatabasePreprocessingStepDefaultTypeInternal {
  PROTOBUF_CONSTEXPR DatabasePreprocessingStepDefaultTypeInternal()
      : _instance(::_pbi::ConstantInitialized{}) {}
  ~DatabasePreprocessingStepDefaultTypeInternal() {}
  union {
    DatabasePreprocessingStep _instance;
  };
};
PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 DatabasePreprocessingStepDefaultTypeInternal _DatabasePreprocessingStep_default_instance_;
PROTOBUF_CONSTEXPR DBFilterByDifficulty::DBFilterByDifficulty(
    ::_pbi::ConstantInitialized): _impl_{
    /*decltype(_impl_.removed_difficulties_)*/{}
  , /*decltype(_impl_._removed_difficulties_cached_byte_size_)*/{0}
  , /*decltype(_impl_._cached_size_)*/{}} {}
struct DBFilterByDifficultyDefaultTypeInternal {
  PROTOBUF_CONSTEXPR DBFilterByDifficultyDefaultTypeInternal()
      : _instance(::_pbi::ConstantInitialized{}) {}
  ~DBFilterByDifficultyDefaultTypeInternal() {}
  union {
    DBFilterByDifficulty _instance;
  };
};
PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 DBFilterByDifficultyDefaultTypeInternal _DBFilterByDifficulty_default_instance_;
PROTOBUF_CONSTEXPR DBFilterByMinNumPointInGroundTruth_MinNumPointPairsEntry_DoNotUse::DBFilterByMinNumPointInGroundTruth_MinNumPointPairsEntry_DoNotUse(
    ::_pbi::ConstantInitialized) {}
struct DBFilterByMinNumPointInGroundTruth_MinNumPointPairsEntry_DoNotUseDefaultTypeInternal {
  PROTOBUF_CONSTEXPR DBFilterByMinNumPointInGroundTruth_MinNumPointPairsEntry_DoNotUseDefaultTypeInternal()
      : _instance(::_pbi::ConstantInitialized{}) {}
  ~DBFilterByMinNumPointInGroundTruth_MinNumPointPairsEntry_DoNotUseDefaultTypeInternal() {}
  union {
    DBFilterByMinNumPointInGroundTruth_MinNumPointPairsEntry_DoNotUse _instance;
  };
};
PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 DBFilterByMinNumPointInGroundTruth_MinNumPointPairsEntry_DoNotUseDefaultTypeInternal _DBFilterByMinNumPointInGroundTruth_MinNumPointPairsEntry_DoNotUse_default_instance_;
PROTOBUF_CONSTEXPR DBFilterByMinNumPointInGroundTruth::DBFilterByMinNumPointInGroundTruth(
    ::_pbi::ConstantInitialized): _impl_{
    /*decltype(_impl_.min_num_point_pairs_)*/{::_pbi::ConstantInitialized()}
  , /*decltype(_impl_._cached_size_)*/{}} {}
struct DBFilterByMinNumPointInGroundTruthDefaultTypeInternal {
  PROTOBUF_CONSTEXPR DBFilterByMinNumPointInGroundTruthDefaultTypeInternal()
      : _instance(::_pbi::ConstantInitialized{}) {}
  ~DBFilterByMinNumPointInGroundTruthDefaultTypeInternal() {}
  union {
    DBFilterByMinNumPointInGroundTruth _instance;
  };
};
PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 DBFilterByMinNumPointInGroundTruthDefaultTypeInternal _DBFilterByMinNumPointInGroundTruth_default_instance_;
PROTOBUF_CONSTEXPR PreprocessingStep::PreprocessingStep(
    ::_pbi::ConstantInitialized): _impl_{
    /*decltype(_impl_.preprocessing_step_)*/{}
  , /*decltype(_impl_._cached_size_)*/{}
  , /*decltype(_impl_._oneof_case_)*/{}} {}
struct PreprocessingStepDefaultTypeInternal {
  PROTOBUF_CONSTEXPR PreprocessingStepDefaultTypeInternal()
      : _instance(::_pbi::ConstantInitialized{}) {}
  ~PreprocessingStepDefaultTypeInternal() {}
  union {
    PreprocessingStep _instance;
  };
};
PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PreprocessingStepDefaultTypeInternal _PreprocessingStep_default_instance_;
PROTOBUF_CONSTEXPR RandomGlobalScaling::RandomGlobalScaling(
    ::_pbi::ConstantInitialized): _impl_{
    /*decltype(_impl_.min_scale_)*/0
  , /*decltype(_impl_.max_scale_)*/0
  , /*decltype(_impl_._cached_size_)*/{}} {}
struct RandomGlobalScalingDefaultTypeInternal {
  PROTOBUF_CONSTEXPR RandomGlobalScalingDefaultTypeInternal()
      : _instance(::_pbi::ConstantInitialized{}) {}
  ~RandomGlobalScalingDefaultTypeInternal() {}
  union {
    RandomGlobalScaling _instance;
  };
};
PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 RandomGlobalScalingDefaultTypeInternal _RandomGlobalScaling_default_instance_;
PROTOBUF_CONSTEXPR RandomGlobalRotation::RandomGlobalRotation(
    ::_pbi::ConstantInitialized): _impl_{
    /*decltype(_impl_.min_rad_)*/0
  , /*decltype(_impl_.max_rad_)*/0
  , /*decltype(_impl_._cached_size_)*/{}} {}
struct RandomGlobalRotationDefaultTypeInternal {
  PROTOBUF_CONSTEXPR RandomGlobalRotationDefaultTypeInternal()
      : _instance(::_pbi::ConstantInitialized{}) {}
  ~RandomGlobalRotationDefaultTypeInternal() {}
  union {
    RandomGlobalRotation _instance;
  };
};
PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 RandomGlobalRotationDefaultTypeInternal _RandomGlobalRotation_default_instance_;
}  // namespace protos
}  // namespace second
static ::_pb::Metadata file_level_metadata_second_2fprotos_2fpreprocess_2eproto[8];
static constexpr ::_pb::EnumDescriptor const** file_level_enum_descriptors_second_2fprotos_2fpreprocess_2eproto = nullptr;
static constexpr ::_pb::ServiceDescriptor const** file_level_service_descriptors_second_2fprotos_2fpreprocess_2eproto = nullptr;

const uint32_t TableStruct_second_2fprotos_2fpreprocess_2eproto::offsets[] PROTOBUF_SECTION_VARIABLE(protodesc_cold) = {
  ~0u,  // no _has_bits_
  PROTOBUF_FIELD_OFFSET(::second::protos::Preprocess, _internal_metadata_),
  ~0u,  // no _extensions_
  ~0u,  // no _oneof_case_
  ~0u,  // no _weak_field_map_
  ~0u,  // no _inlined_string_donated_
  PROTOBUF_FIELD_OFFSET(::second::protos::Preprocess, _impl_.random_global_rotation_min_rad_),
  PROTOBUF_FIELD_OFFSET(::second::protos::Preprocess, _impl_.random_global_rotation_max_rad_),
  PROTOBUF_FIELD_OFFSET(::second::protos::Preprocess, _impl_.random_global_scaling_min_),
  PROTOBUF_FIELD_OFFSET(::second::protos::Preprocess, _impl_.random_global_scaling_max_),
  PROTOBUF_FIELD_OFFSET(::second::protos::Preprocess, _impl_.random_noise_per_groundtruth_min_rad_),
  PROTOBUF_FIELD_OFFSET(::second::protos::Preprocess, _impl_.random_noise_per_groundtruth_max_rad_),
  PROTOBUF_FIELD_OFFSET(::second::protos::Preprocess, _impl_.random_noise_per_groundtruth_position_std_),
  ~0u,  // no _has_bits_
  PROTOBUF_FIELD_OFFSET(::second::protos::DatabasePreprocessingStep, _internal_metadata_),
  ~0u,  // no _extensions_
  PROTOBUF_FIELD_OFFSET(::second::protos::DatabasePreprocessingStep, _impl_._oneof_case_[0]),
  ~0u,  // no _weak_field_map_
  ~0u,  // no _inlined_string_donated_
  ::_pbi::kInvalidFieldOffsetTag,
  ::_pbi::kInvalidFieldOffsetTag,
  PROTOBUF_FIELD_OFFSET(::second::protos::DatabasePreprocessingStep, _impl_.database_preprocessing_step_),
  ~0u,  // no _has_bits_
  PROTOBUF_FIELD_OFFSET(::second::protos::DBFilterByDifficulty, _internal_metadata_),
  ~0u,  // no _extensions_
  ~0u,  // no _oneof_case_
  ~0u,  // no _weak_field_map_
  ~0u,  // no _inlined_string_donated_
  PROTOBUF_FIELD_OFFSET(::second::protos::DBFilterByDifficulty, _impl_.removed_difficulties_),
  PROTOBUF_FIELD_OFFSET(::second::protos::DBFilterByMinNumPointInGroundTruth_MinNumPointPairsEntry_DoNotUse, _has_bits_),
  PROTOBUF_FIELD_OFFSET(::second::protos::DBFilterByMinNumPointInGroundTruth_MinNumPointPairsEntry_DoNotUse, _internal_metadata_),
  ~0u,  // no _extensions_
  ~0u,  // no _oneof_case_
  ~0u,  // no _weak_field_map_
  ~0u,  // no _inlined_string_donated_
  PROTOBUF_FIELD_OFFSET(::second::protos::DBFilterByMinNumPointInGroundTruth_MinNumPointPairsEntry_DoNotUse, key_),
  PROTOBUF_FIELD_OFFSET(::second::protos::DBFilterByMinNumPointInGroundTruth_MinNumPointPairsEntry_DoNotUse, value_),
  0,
  1,
  ~0u,  // no _has_bits_
  PROTOBUF_FIELD_OFFSET(::second::protos::DBFilterByMinNumPointInGroundTruth, _internal_metadata_),
  ~0u,  // no _extensions_
  ~0u,  // no _oneof_case_
  ~0u,  // no _weak_field_map_
  ~0u,  // no _inlined_string_donated_
  PROTOBUF_FIELD_OFFSET(::second::protos::DBFilterByMinNumPointInGroundTruth, _impl_.min_num_point_pairs_),
  ~0u,  // no _has_bits_
  PROTOBUF_FIELD_OFFSET(::second::protos::PreprocessingStep, _internal_metadata_),
  ~0u,  // no _extensions_
  PROTOBUF_FIELD_OFFSET(::second::protos::PreprocessingStep, _impl_._oneof_case_[0]),
  ~0u,  // no _weak_field_map_
  ~0u,  // no _inlined_string_donated_
  ::_pbi::kInvalidFieldOffsetTag,
  ::_pbi::kInvalidFieldOffsetTag,
  PROTOBUF_FIELD_OFFSET(::second::protos::PreprocessingStep, _impl_.preprocessing_step_),
  ~0u,  // no _has_bits_
  PROTOBUF_FIELD_OFFSET(::second::protos::RandomGlobalScaling, _internal_metadata_),
  ~0u,  // no _extensions_
  ~0u,  // no _oneof_case_
  ~0u,  // no _weak_field_map_
  ~0u,  // no _inlined_string_donated_
  PROTOBUF_FIELD_OFFSET(::second::protos::RandomGlobalScaling, _impl_.min_scale_),
  PROTOBUF_FIELD_OFFSET(::second::protos::RandomGlobalScaling, _impl_.max_scale_),
  ~0u,  // no _has_bits_
  PROTOBUF_FIELD_OFFSET(::second::protos::RandomGlobalRotation, _internal_metadata_),
  ~0u,  // no _extensions_
  ~0u,  // no _oneof_case_
  ~0u,  // no _weak_field_map_
  ~0u,  // no _inlined_string_donated_
  PROTOBUF_FIELD_OFFSET(::second::protos::RandomGlobalRotation, _impl_.min_rad_),
  PROTOBUF_FIELD_OFFSET(::second::protos::RandomGlobalRotation, _impl_.max_rad_),
};
static const ::_pbi::MigrationSchema schemas[] PROTOBUF_SECTION_VARIABLE(protodesc_cold) = {
  { 0, -1, -1, sizeof(::second::protos::Preprocess)},
  { 13, -1, -1, sizeof(::second::protos::DatabasePreprocessingStep)},
  { 22, -1, -1, sizeof(::second::protos::DBFilterByDifficulty)},
  { 29, 37, -1, sizeof(::second::protos::DBFilterByMinNumPointInGroundTruth_MinNumPointPairsEntry_DoNotUse)},
  { 39, -1, -1, sizeof(::second::protos::DBFilterByMinNumPointInGroundTruth)},
  { 46, -1, -1, sizeof(::second::protos::PreprocessingStep)},
  { 55, -1, -1, sizeof(::second::protos::RandomGlobalScaling)},
  { 63, -1, -1, sizeof(::second::protos::RandomGlobalRotation)},
};

static const ::_pb::Message* const file_default_instances[] = {
  &::second::protos::_Preprocess_default_instance_._instance,
  &::second::protos::_DatabasePreprocessingStep_default_instance_._instance,
  &::second::protos::_DBFilterByDifficulty_default_instance_._instance,
  &::second::protos::_DBFilterByMinNumPointInGroundTruth_MinNumPointPairsEntry_DoNotUse_default_instance_._instance,
  &::second::protos::_DBFilterByMinNumPointInGroundTruth_default_instance_._instance,
  &::second::protos::_PreprocessingStep_default_instance_._instance,
  &::second::protos::_RandomGlobalScaling_default_instance_._instance,
  &::second::protos::_RandomGlobalRotation_default_instance_._instance,
};

const char descriptor_table_protodef_second_2fprotos_2fpreprocess_2eproto[] PROTOBUF_SECTION_VARIABLE(protodesc_cold) =
  "\n\036second/protos/preprocess.proto\022\rsecond"
  ".protos\"\261\002\n\nPreprocess\022&\n\036random_global_"
  "rotation_min_rad\030\001 \001(\002\022&\n\036random_global_"
  "rotation_max_rad\030\002 \001(\002\022!\n\031random_global_"
  "scaling_min\030\003 \001(\002\022!\n\031random_global_scali"
  "ng_max\030\004 \001(\002\022,\n$random_noise_per_groundt"
  "ruth_min_rad\030\005 \001(\002\022,\n$random_noise_per_g"
  "roundtruth_max_rad\030\006 \001(\002\0221\n)random_noise"
  "_per_groundtruth_position_std\030\007 \001(\002\"\326\001\n\031"
  "DatabasePreprocessingStep\022C\n\024filter_by_d"
  "ifficulty\030\001 \001(\0132#.second.protos.DBFilter"
  "ByDifficultyH\000\022U\n\030filter_by_min_num_poin"
  "ts\030\002 \001(\01321.second.protos.DBFilterByMinNu"
  "mPointInGroundTruthH\000B\035\n\033database_prepro"
  "cessing_step\"4\n\024DBFilterByDifficulty\022\034\n\024"
  "removed_difficulties\030\001 \003(\005\"\303\001\n\"DBFilterB"
  "yMinNumPointInGroundTruth\022d\n\023min_num_poi"
  "nt_pairs\030\001 \003(\0132G.second.protos.DBFilterB"
  "yMinNumPointInGroundTruth.MinNumPointPai"
  "rsEntry\0327\n\025MinNumPointPairsEntry\022\013\n\003key\030"
  "\001 \001(\t\022\r\n\005value\030\002 \001(\r:\0028\001\"\265\001\n\021Preprocessi"
  "ngStep\022C\n\025random_global_scaling\030\001 \001(\0132\"."
  "second.protos.RandomGlobalScalingH\000\022E\n\026r"
  "andom_global_rotation\030\002 \001(\0132#.second.pro"
  "tos.RandomGlobalRotationH\000B\024\n\022preprocess"
  "ing_step\";\n\023RandomGlobalScaling\022\021\n\tmin_s"
  "cale\030\001 \001(\002\022\021\n\tmax_scale\030\002 \001(\002\"8\n\024RandomG"
  "lobalRotation\022\017\n\007min_rad\030\001 \001(\002\022\017\n\007max_ra"
  "d\030\002 \001(\002b\006proto3"
  ;
static ::_pbi::once_flag descriptor_table_second_2fprotos_2fpreprocess_2eproto_once;
const ::_pbi::DescriptorTable descriptor_table_second_2fprotos_2fpreprocess_2eproto = {
    false, false, 1135, descriptor_table_protodef_second_2fprotos_2fpreprocess_2eproto,
    "second/protos/preprocess.proto",
    &descriptor_table_second_2fprotos_2fpreprocess_2eproto_once, nullptr, 0, 8,
    schemas, file_default_instances, TableStruct_second_2fprotos_2fpreprocess_2eproto::offsets,
    file_level_metadata_second_2fprotos_2fpreprocess_2eproto, file_level_enum_descriptors_second_2fprotos_2fpreprocess_2eproto,
    file_level_service_descriptors_second_2fprotos_2fpreprocess_2eproto,
};
PROTOBUF_ATTRIBUTE_WEAK const ::_pbi::DescriptorTable* descriptor_table_second_2fprotos_2fpreprocess_2eproto_getter() {
  return &descriptor_table_second_2fprotos_2fpreprocess_2eproto;
}

// Force running AddDescriptors() at dynamic initialization time.
PROTOBUF_ATTRIBUTE_INIT_PRIORITY2 static ::_pbi::AddDescriptorsRunner dynamic_init_dummy_second_2fprotos_2fpreprocess_2eproto(&descriptor_table_second_2fprotos_2fpreprocess_2eproto);
namespace second {
namespace protos {

// ===================================================================

class Preprocess::_Internal {
 public:
};

Preprocess::Preprocess(::PROTOBUF_NAMESPACE_ID::Arena* arena,
                         bool is_message_owned)
  : ::PROTOBUF_NAMESPACE_ID::Message(arena, is_message_owned) {
  SharedCtor(arena, is_message_owned);
  // @@protoc_insertion_point(arena_constructor:second.protos.Preprocess)
}
Preprocess::Preprocess(const Preprocess& from)
  : ::PROTOBUF_NAMESPACE_ID::Message() {
  Preprocess* const _this = this; (void)_this;
  new (&_impl_) Impl_{
      decltype(_impl_.random_global_rotation_min_rad_){}
    , decltype(_impl_.random_global_rotation_max_rad_){}
    , decltype(_impl_.random_global_scaling_min_){}
    , decltype(_impl_.random_global_scaling_max_){}
    , decltype(_impl_.random_noise_per_groundtruth_min_rad_){}
    , decltype(_impl_.random_noise_per_groundtruth_max_rad_){}
    , decltype(_impl_.random_noise_per_groundtruth_position_std_){}
    , /*decltype(_impl_._cached_size_)*/{}};

  _internal_metadata_.MergeFrom<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(from._internal_metadata_);
  ::memcpy(&_impl_.random_global_rotation_min_rad_, &from._impl_.random_global_rotation_min_rad_,
    static_cast<size_t>(reinterpret_cast<char*>(&_impl_.random_noise_per_groundtruth_position_std_) -
    reinterpret_cast<char*>(&_impl_.random_global_rotation_min_rad_)) + sizeof(_impl_.random_noise_per_groundtruth_position_std_));
  // @@protoc_insertion_point(copy_constructor:second.protos.Preprocess)
}

inline void Preprocess::SharedCtor(
    ::_pb::Arena* arena, bool is_message_owned) {
  (void)arena;
  (void)is_message_owned;
  new (&_impl_) Impl_{
      decltype(_impl_.random_global_rotation_min_rad_){0}
    , decltype(_impl_.random_global_rotation_max_rad_){0}
    , decltype(_impl_.random_global_scaling_min_){0}
    , decltype(_impl_.random_global_scaling_max_){0}
    , decltype(_impl_.random_noise_per_groundtruth_min_rad_){0}
    , decltype(_impl_.random_noise_per_groundtruth_max_rad_){0}
    , decltype(_impl_.random_noise_per_groundtruth_position_std_){0}
    , /*decltype(_impl_._cached_size_)*/{}
  };
}

Preprocess::~Preprocess() {
  // @@protoc_insertion_point(destructor:second.protos.Preprocess)
  if (auto *arena = _internal_metadata_.DeleteReturnArena<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>()) {
  (void)arena;
    return;
  }
  SharedDtor();
}

inline void Preprocess::SharedDtor() {
  GOOGLE_DCHECK(GetArenaForAllocation() == nullptr);
}

void Preprocess::SetCachedSize(int size) const {
  _impl_._cached_size_.Set(size);
}

void Preprocess::Clear() {
// @@protoc_insertion_point(message_clear_start:second.protos.Preprocess)
  uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  ::memset(&_impl_.random_global_rotation_min_rad_, 0, static_cast<size_t>(
      reinterpret_cast<char*>(&_impl_.random_noise_per_groundtruth_position_std_) -
      reinterpret_cast<char*>(&_impl_.random_global_rotation_min_rad_)) + sizeof(_impl_.random_noise_per_groundtruth_position_std_));
  _internal_metadata_.Clear<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>();
}

const char* Preprocess::_InternalParse(const char* ptr, ::_pbi::ParseContext* ctx) {
#define CHK_(x) if (PROTOBUF_PREDICT_FALSE(!(x))) goto failure
  while (!ctx->Done(&ptr)) {
    uint32_t tag;
    ptr = ::_pbi::ReadTag(ptr, &tag);
    switch (tag >> 3) {
      // float random_global_rotation_min_rad = 1;
      case 1:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 13)) {
          _impl_.random_global_rotation_min_rad_ = ::PROTOBUF_NAMESPACE_ID::internal::UnalignedLoad<float>(ptr);
          ptr += sizeof(float);
        } else
          goto handle_unusual;
        continue;
      // float random_global_rotation_max_rad = 2;
      case 2:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 21)) {
          _impl_.random_global_rotation_max_rad_ = ::PROTOBUF_NAMESPACE_ID::internal::UnalignedLoad<float>(ptr);
          ptr += sizeof(float);
        } else
          goto handle_unusual;
        continue;
      // float random_global_scaling_min = 3;
      case 3:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 29)) {
          _impl_.random_global_scaling_min_ = ::PROTOBUF_NAMESPACE_ID::internal::UnalignedLoad<float>(ptr);
          ptr += sizeof(float);
        } else
          goto handle_unusual;
        continue;
      // float random_global_scaling_max = 4;
      case 4:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 37)) {
          _impl_.random_global_scaling_max_ = ::PROTOBUF_NAMESPACE_ID::internal::UnalignedLoad<float>(ptr);
          ptr += sizeof(float);
        } else
          goto handle_unusual;
        continue;
      // float random_noise_per_groundtruth_min_rad = 5;
      case 5:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 45)) {
          _impl_.random_noise_per_groundtruth_min_rad_ = ::PROTOBUF_NAMESPACE_ID::internal::UnalignedLoad<float>(ptr);
          ptr += sizeof(float);
        } else
          goto handle_unusual;
        continue;
      // float random_noise_per_groundtruth_max_rad = 6;
      case 6:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 53)) {
          _impl_.random_noise_per_groundtruth_max_rad_ = ::PROTOBUF_NAMESPACE_ID::internal::UnalignedLoad<float>(ptr);
          ptr += sizeof(float);
        } else
          goto handle_unusual;
        continue;
      // float random_noise_per_groundtruth_position_std = 7;
      case 7:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 61)) {
          _impl_.random_noise_per_groundtruth_position_std_ = ::PROTOBUF_NAMESPACE_ID::internal::UnalignedLoad<float>(ptr);
          ptr += sizeof(float);
        } else
          goto handle_unusual;
        continue;
      default:
        goto handle_unusual;
    }  // switch
  handle_unusual:
    if ((tag == 0) || ((tag & 7) == 4)) {
      CHK_(ptr);
      ctx->SetLastTag(tag);
      goto message_done;
    }
    ptr = UnknownFieldParse(
        tag,
        _internal_metadata_.mutable_unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(),
        ptr, ctx);
    CHK_(ptr != nullptr);
  }  // while
message_done:
  return ptr;
failure:
  ptr = nullptr;
  goto message_done;
#undef CHK_
}

uint8_t* Preprocess::_InternalSerialize(
    uint8_t* target, ::PROTOBUF_NAMESPACE_ID::io::EpsCopyOutputStream* stream) const {
  // @@protoc_insertion_point(serialize_to_array_start:second.protos.Preprocess)
  uint32_t cached_has_bits = 0;
  (void) cached_has_bits;

  // float random_global_rotation_min_rad = 1;
  static_assert(sizeof(uint32_t) == sizeof(float), "Code assumes uint32_t and float are the same size.");
  float tmp_random_global_rotation_min_rad = this->_internal_random_global_rotation_min_rad();
  uint32_t raw_random_global_rotation_min_rad;
  memcpy(&raw_random_global_rotation_min_rad, &tmp_random_global_rotation_min_rad, sizeof(tmp_random_global_rotation_min_rad));
  if (raw_random_global_rotation_min_rad != 0) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteFloatToArray(1, this->_internal_random_global_rotation_min_rad(), target);
  }

  // float random_global_rotation_max_rad = 2;
  static_assert(sizeof(uint32_t) == sizeof(float), "Code assumes uint32_t and float are the same size.");
  float tmp_random_global_rotation_max_rad = this->_internal_random_global_rotation_max_rad();
  uint32_t raw_random_global_rotation_max_rad;
  memcpy(&raw_random_global_rotation_max_rad, &tmp_random_global_rotation_max_rad, sizeof(tmp_random_global_rotation_max_rad));
  if (raw_random_global_rotation_max_rad != 0) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteFloatToArray(2, this->_internal_random_global_rotation_max_rad(), target);
  }

  // float random_global_scaling_min = 3;
  static_assert(sizeof(uint32_t) == sizeof(float), "Code assumes uint32_t and float are the same size.");
  float tmp_random_global_scaling_min = this->_internal_random_global_scaling_min();
  uint32_t raw_random_global_scaling_min;
  memcpy(&raw_random_global_scaling_min, &tmp_random_global_scaling_min, sizeof(tmp_random_global_scaling_min));
  if (raw_random_global_scaling_min != 0) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteFloatToArray(3, this->_internal_random_global_scaling_min(), target);
  }

  // float random_global_scaling_max = 4;
  static_assert(sizeof(uint32_t) == sizeof(float), "Code assumes uint32_t and float are the same size.");
  float tmp_random_global_scaling_max = this->_internal_random_global_scaling_max();
  uint32_t raw_random_global_scaling_max;
  memcpy(&raw_random_global_scaling_max, &tmp_random_global_scaling_max, sizeof(tmp_random_global_scaling_max));
  if (raw_random_global_scaling_max != 0) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteFloatToArray(4, this->_internal_random_global_scaling_max(), target);
  }

  // float random_noise_per_groundtruth_min_rad = 5;
  static_assert(sizeof(uint32_t) == sizeof(float), "Code assumes uint32_t and float are the same size.");
  float tmp_random_noise_per_groundtruth_min_rad = this->_internal_random_noise_per_groundtruth_min_rad();
  uint32_t raw_random_noise_per_groundtruth_min_rad;
  memcpy(&raw_random_noise_per_groundtruth_min_rad, &tmp_random_noise_per_groundtruth_min_rad, sizeof(tmp_random_noise_per_groundtruth_min_rad));
  if (raw_random_noise_per_groundtruth_min_rad != 0) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteFloatToArray(5, this->_internal_random_noise_per_groundtruth_min_rad(), target);
  }

  // float random_noise_per_groundtruth_max_rad = 6;
  static_assert(sizeof(uint32_t) == sizeof(float), "Code assumes uint32_t and float are the same size.");
  float tmp_random_noise_per_groundtruth_max_rad = this->_internal_random_noise_per_groundtruth_max_rad();
  uint32_t raw_random_noise_per_groundtruth_max_rad;
  memcpy(&raw_random_noise_per_groundtruth_max_rad, &tmp_random_noise_per_groundtruth_max_rad, sizeof(tmp_random_noise_per_groundtruth_max_rad));
  if (raw_random_noise_per_groundtruth_max_rad != 0) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteFloatToArray(6, this->_internal_random_noise_per_groundtruth_max_rad(), target);
  }

  // float random_noise_per_groundtruth_position_std = 7;
  static_assert(sizeof(uint32_t) == sizeof(float), "Code assumes uint32_t and float are the same size.");
  float tmp_random_noise_per_groundtruth_position_std = this->_internal_random_noise_per_groundtruth_position_std();
  uint32_t raw_random_noise_per_groundtruth_position_std;
  memcpy(&raw_random_noise_per_groundtruth_position_std, &tmp_random_noise_per_groundtruth_position_std, sizeof(tmp_random_noise_per_groundtruth_position_std));
  if (raw_random_noise_per_groundtruth_position_std != 0) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteFloatToArray(7, this->_internal_random_noise_per_groundtruth_position_std(), target);
  }

  if (PROTOBUF_PREDICT_FALSE(_internal_metadata_.have_unknown_fields())) {
    target = ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
        _internal_metadata_.unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(::PROTOBUF_NAMESPACE_ID::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:second.protos.Preprocess)
  return target;
}

size_t Preprocess::ByteSizeLong() const {
// @@protoc_insertion_point(message_byte_size_start:second.protos.Preprocess)
  size_t total_size = 0;

  uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  // float random_global_rotation_min_rad = 1;
  static_assert(sizeof(uint32_t) == sizeof(float), "Code assumes uint32_t and float are the same size.");
  float tmp_random_global_rotation_min_rad = this->_internal_random_global_rotation_min_rad();
  uint32_t raw_random_global_rotation_min_rad;
  memcpy(&raw_random_global_rotation_min_rad, &tmp_random_global_rotation_min_rad, sizeof(tmp_random_global_rotation_min_rad));
  if (raw_random_global_rotation_min_rad != 0) {
    total_size += 1 + 4;
  }

  // float random_global_rotation_max_rad = 2;
  static_assert(sizeof(uint32_t) == sizeof(float), "Code assumes uint32_t and float are the same size.");
  float tmp_random_global_rotation_max_rad = this->_internal_random_global_rotation_max_rad();
  uint32_t raw_random_global_rotation_max_rad;
  memcpy(&raw_random_global_rotation_max_rad, &tmp_random_global_rotation_max_rad, sizeof(tmp_random_global_rotation_max_rad));
  if (raw_random_global_rotation_max_rad != 0) {
    total_size += 1 + 4;
  }

  // float random_global_scaling_min = 3;
  static_assert(sizeof(uint32_t) == sizeof(float), "Code assumes uint32_t and float are the same size.");
  float tmp_random_global_scaling_min = this->_internal_random_global_scaling_min();
  uint32_t raw_random_global_scaling_min;
  memcpy(&raw_random_global_scaling_min, &tmp_random_global_scaling_min, sizeof(tmp_random_global_scaling_min));
  if (raw_random_global_scaling_min != 0) {
    total_size += 1 + 4;
  }

  // float random_global_scaling_max = 4;
  static_assert(sizeof(uint32_t) == sizeof(float), "Code assumes uint32_t and float are the same size.");
  float tmp_random_global_scaling_max = this->_internal_random_global_scaling_max();
  uint32_t raw_random_global_scaling_max;
  memcpy(&raw_random_global_scaling_max, &tmp_random_global_scaling_max, sizeof(tmp_random_global_scaling_max));
  if (raw_random_global_scaling_max != 0) {
    total_size += 1 + 4;
  }

  // float random_noise_per_groundtruth_min_rad = 5;
  static_assert(sizeof(uint32_t) == sizeof(float), "Code assumes uint32_t and float are the same size.");
  float tmp_random_noise_per_groundtruth_min_rad = this->_internal_random_noise_per_groundtruth_min_rad();
  uint32_t raw_random_noise_per_groundtruth_min_rad;
  memcpy(&raw_random_noise_per_groundtruth_min_rad, &tmp_random_noise_per_groundtruth_min_rad, sizeof(tmp_random_noise_per_groundtruth_min_rad));
  if (raw_random_noise_per_groundtruth_min_rad != 0) {
    total_size += 1 + 4;
  }

  // float random_noise_per_groundtruth_max_rad = 6;
  static_assert(sizeof(uint32_t) == sizeof(float), "Code assumes uint32_t and float are the same size.");
  float tmp_random_noise_per_groundtruth_max_rad = this->_internal_random_noise_per_groundtruth_max_rad();
  uint32_t raw_random_noise_per_groundtruth_max_rad;
  memcpy(&raw_random_noise_per_groundtruth_max_rad, &tmp_random_noise_per_groundtruth_max_rad, sizeof(tmp_random_noise_per_groundtruth_max_rad));
  if (raw_random_noise_per_groundtruth_max_rad != 0) {
    total_size += 1 + 4;
  }

  // float random_noise_per_groundtruth_position_std = 7;
  static_assert(sizeof(uint32_t) == sizeof(float), "Code assumes uint32_t and float are the same size.");
  float tmp_random_noise_per_groundtruth_position_std = this->_internal_random_noise_per_groundtruth_position_std();
  uint32_t raw_random_noise_per_groundtruth_position_std;
  memcpy(&raw_random_noise_per_groundtruth_position_std, &tmp_random_noise_per_groundtruth_position_std, sizeof(tmp_random_noise_per_groundtruth_position_std));
  if (raw_random_noise_per_groundtruth_position_std != 0) {
    total_size += 1 + 4;
  }

  return MaybeComputeUnknownFieldsSize(total_size, &_impl_._cached_size_);
}

const ::PROTOBUF_NAMESPACE_ID::Message::ClassData Preprocess::_class_data_ = {
    ::PROTOBUF_NAMESPACE_ID::Message::CopyWithSourceCheck,
    Preprocess::MergeImpl
};
const ::PROTOBUF_NAMESPACE_ID::Message::ClassData*Preprocess::GetClassData() const { return &_class_data_; }


void Preprocess::MergeImpl(::PROTOBUF_NAMESPACE_ID::Message& to_msg, const ::PROTOBUF_NAMESPACE_ID::Message& from_msg) {
  auto* const _this = static_cast<Preprocess*>(&to_msg);
  auto& from = static_cast<const Preprocess&>(from_msg);
  // @@protoc_insertion_point(class_specific_merge_from_start:second.protos.Preprocess)
  GOOGLE_DCHECK_NE(&from, _this);
  uint32_t cached_has_bits = 0;
  (void) cached_has_bits;

  static_assert(sizeof(uint32_t) == sizeof(float), "Code assumes uint32_t and float are the same size.");
  float tmp_random_global_rotation_min_rad = from._internal_random_global_rotation_min_rad();
  uint32_t raw_random_global_rotation_min_rad;
  memcpy(&raw_random_global_rotation_min_rad, &tmp_random_global_rotation_min_rad, sizeof(tmp_random_global_rotation_min_rad));
  if (raw_random_global_rotation_min_rad != 0) {
    _this->_internal_set_random_global_rotation_min_rad(from._internal_random_global_rotation_min_rad());
  }
  static_assert(sizeof(uint32_t) == sizeof(float), "Code assumes uint32_t and float are the same size.");
  float tmp_random_global_rotation_max_rad = from._internal_random_global_rotation_max_rad();
  uint32_t raw_random_global_rotation_max_rad;
  memcpy(&raw_random_global_rotation_max_rad, &tmp_random_global_rotation_max_rad, sizeof(tmp_random_global_rotation_max_rad));
  if (raw_random_global_rotation_max_rad != 0) {
    _this->_internal_set_random_global_rotation_max_rad(from._internal_random_global_rotation_max_rad());
  }
  static_assert(sizeof(uint32_t) == sizeof(float), "Code assumes uint32_t and float are the same size.");
  float tmp_random_global_scaling_min = from._internal_random_global_scaling_min();
  uint32_t raw_random_global_scaling_min;
  memcpy(&raw_random_global_scaling_min, &tmp_random_global_scaling_min, sizeof(tmp_random_global_scaling_min));
  if (raw_random_global_scaling_min != 0) {
    _this->_internal_set_random_global_scaling_min(from._internal_random_global_scaling_min());
  }
  static_assert(sizeof(uint32_t) == sizeof(float), "Code assumes uint32_t and float are the same size.");
  float tmp_random_global_scaling_max = from._internal_random_global_scaling_max();
  uint32_t raw_random_global_scaling_max;
  memcpy(&raw_random_global_scaling_max, &tmp_random_global_scaling_max, sizeof(tmp_random_global_scaling_max));
  if (raw_random_global_scaling_max != 0) {
    _this->_internal_set_random_global_scaling_max(from._internal_random_global_scaling_max());
  }
  static_assert(sizeof(uint32_t) == sizeof(float), "Code assumes uint32_t and float are the same size.");
  float tmp_random_noise_per_groundtruth_min_rad = from._internal_random_noise_per_groundtruth_min_rad();
  uint32_t raw_random_noise_per_groundtruth_min_rad;
  memcpy(&raw_random_noise_per_groundtruth_min_rad, &tmp_random_noise_per_groundtruth_min_rad, sizeof(tmp_random_noise_per_groundtruth_min_rad));
  if (raw_random_noise_per_groundtruth_min_rad != 0) {
    _this->_internal_set_random_noise_per_groundtruth_min_rad(from._internal_random_noise_per_groundtruth_min_rad());
  }
  static_assert(sizeof(uint32_t) == sizeof(float), "Code assumes uint32_t and float are the same size.");
  float tmp_random_noise_per_groundtruth_max_rad = from._internal_random_noise_per_groundtruth_max_rad();
  uint32_t raw_random_noise_per_groundtruth_max_rad;
  memcpy(&raw_random_noise_per_groundtruth_max_rad, &tmp_random_noise_per_groundtruth_max_rad, sizeof(tmp_random_noise_per_groundtruth_max_rad));
  if (raw_random_noise_per_groundtruth_max_rad != 0) {
    _this->_internal_set_random_noise_per_groundtruth_max_rad(from._internal_random_noise_per_groundtruth_max_rad());
  }
  static_assert(sizeof(uint32_t) == sizeof(float), "Code assumes uint32_t and float are the same size.");
  float tmp_random_noise_per_groundtruth_position_std = from._internal_random_noise_per_groundtruth_position_std();
  uint32_t raw_random_noise_per_groundtruth_position_std;
  memcpy(&raw_random_noise_per_groundtruth_position_std, &tmp_random_noise_per_groundtruth_position_std, sizeof(tmp_random_noise_per_groundtruth_position_std));
  if (raw_random_noise_per_groundtruth_position_std != 0) {
    _this->_internal_set_random_noise_per_groundtruth_position_std(from._internal_random_noise_per_groundtruth_position_std());
  }
  _this->_internal_metadata_.MergeFrom<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(from._internal_metadata_);
}

void Preprocess::CopyFrom(const Preprocess& from) {
// @@protoc_insertion_point(class_specific_copy_from_start:second.protos.Preprocess)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}

bool Preprocess::IsInitialized() const {
  return true;
}

void Preprocess::InternalSwap(Preprocess* other) {
  using std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  ::PROTOBUF_NAMESPACE_ID::internal::memswap<
      PROTOBUF_FIELD_OFFSET(Preprocess, _impl_.random_noise_per_groundtruth_position_std_)
      + sizeof(Preprocess::_impl_.random_noise_per_groundtruth_position_std_)
      - PROTOBUF_FIELD_OFFSET(Preprocess, _impl_.random_global_rotation_min_rad_)>(
          reinterpret_cast<char*>(&_impl_.random_global_rotation_min_rad_),
          reinterpret_cast<char*>(&other->_impl_.random_global_rotation_min_rad_));
}

::PROTOBUF_NAMESPACE_ID::Metadata Preprocess::GetMetadata() const {
  return ::_pbi::AssignDescriptors(
      &descriptor_table_second_2fprotos_2fpreprocess_2eproto_getter, &descriptor_table_second_2fprotos_2fpreprocess_2eproto_once,
      file_level_metadata_second_2fprotos_2fpreprocess_2eproto[0]);
}

// ===================================================================

class DatabasePreprocessingStep::_Internal {
 public:
  static const ::second::protos::DBFilterByDifficulty& filter_by_difficulty(const DatabasePreprocessingStep* msg);
  static const ::second::protos::DBFilterByMinNumPointInGroundTruth& filter_by_min_num_points(const DatabasePreprocessingStep* msg);
};

const ::second::protos::DBFilterByDifficulty&
DatabasePreprocessingStep::_Internal::filter_by_difficulty(const DatabasePreprocessingStep* msg) {
  return *msg->_impl_.database_preprocessing_step_.filter_by_difficulty_;
}
const ::second::protos::DBFilterByMinNumPointInGroundTruth&
DatabasePreprocessingStep::_Internal::filter_by_min_num_points(const DatabasePreprocessingStep* msg) {
  return *msg->_impl_.database_preprocessing_step_.filter_by_min_num_points_;
}
void DatabasePreprocessingStep::set_allocated_filter_by_difficulty(::second::protos::DBFilterByDifficulty* filter_by_difficulty) {
  ::PROTOBUF_NAMESPACE_ID::Arena* message_arena = GetArenaForAllocation();
  clear_database_preprocessing_step();
  if (filter_by_difficulty) {
    ::PROTOBUF_NAMESPACE_ID::Arena* submessage_arena =
      ::PROTOBUF_NAMESPACE_ID::Arena::InternalGetOwningArena(filter_by_difficulty);
    if (message_arena != submessage_arena) {
      filter_by_difficulty = ::PROTOBUF_NAMESPACE_ID::internal::GetOwnedMessage(
          message_arena, filter_by_difficulty, submessage_arena);
    }
    set_has_filter_by_difficulty();
    _impl_.database_preprocessing_step_.filter_by_difficulty_ = filter_by_difficulty;
  }
  // @@protoc_insertion_point(field_set_allocated:second.protos.DatabasePreprocessingStep.filter_by_difficulty)
}
void DatabasePreprocessingStep::set_allocated_filter_by_min_num_points(::second::protos::DBFilterByMinNumPointInGroundTruth* filter_by_min_num_points) {
  ::PROTOBUF_NAMESPACE_ID::Arena* message_arena = GetArenaForAllocation();
  clear_database_preprocessing_step();
  if (filter_by_min_num_points) {
    ::PROTOBUF_NAMESPACE_ID::Arena* submessage_arena =
      ::PROTOBUF_NAMESPACE_ID::Arena::InternalGetOwningArena(filter_by_min_num_points);
    if (message_arena != submessage_arena) {
      filter_by_min_num_points = ::PROTOBUF_NAMESPACE_ID::internal::GetOwnedMessage(
          message_arena, filter_by_min_num_points, submessage_arena);
    }
    set_has_filter_by_min_num_points();
    _impl_.database_preprocessing_step_.filter_by_min_num_points_ = filter_by_min_num_points;
  }
  // @@protoc_insertion_point(field_set_allocated:second.protos.DatabasePreprocessingStep.filter_by_min_num_points)
}
DatabasePreprocessingStep::DatabasePreprocessingStep(::PROTOBUF_NAMESPACE_ID::Arena* arena,
                         bool is_message_owned)
  : ::PROTOBUF_NAMESPACE_ID::Message(arena, is_message_owned) {
  SharedCtor(arena, is_message_owned);
  // @@protoc_insertion_point(arena_constructor:second.protos.DatabasePreprocessingStep)
}
DatabasePreprocessingStep::DatabasePreprocessingStep(const DatabasePreprocessingStep& from)
  : ::PROTOBUF_NAMESPACE_ID::Message() {
  DatabasePreprocessingStep* const _this = this; (void)_this;
  new (&_impl_) Impl_{
      decltype(_impl_.database_preprocessing_step_){}
    , /*decltype(_impl_._cached_size_)*/{}
    , /*decltype(_impl_._oneof_case_)*/{}};

  _internal_metadata_.MergeFrom<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(from._internal_metadata_);
  clear_has_database_preprocessing_step();
  switch (from.database_preprocessing_step_case()) {
    case kFilterByDifficulty: {
      _this->_internal_mutable_filter_by_difficulty()->::second::protos::DBFilterByDifficulty::MergeFrom(
          from._internal_filter_by_difficulty());
      break;
    }
    case kFilterByMinNumPoints: {
      _this->_internal_mutable_filter_by_min_num_points()->::second::protos::DBFilterByMinNumPointInGroundTruth::MergeFrom(
          from._internal_filter_by_min_num_points());
      break;
    }
    case DATABASE_PREPROCESSING_STEP_NOT_SET: {
      break;
    }
  }
  // @@protoc_insertion_point(copy_constructor:second.protos.DatabasePreprocessingStep)
}

inline void DatabasePreprocessingStep::SharedCtor(
    ::_pb::Arena* arena, bool is_message_owned) {
  (void)arena;
  (void)is_message_owned;
  new (&_impl_) Impl_{
      decltype(_impl_.database_preprocessing_step_){}
    , /*decltype(_impl_._cached_size_)*/{}
    , /*decltype(_impl_._oneof_case_)*/{}
  };
  clear_has_database_preprocessing_step();
}

DatabasePreprocessingStep::~DatabasePreprocessingStep() {
  // @@protoc_insertion_point(destructor:second.protos.DatabasePreprocessingStep)
  if (auto *arena = _internal_metadata_.DeleteReturnArena<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>()) {
  (void)arena;
    return;
  }
  SharedDtor();
}

inline void DatabasePreprocessingStep::SharedDtor() {
  GOOGLE_DCHECK(GetArenaForAllocation() == nullptr);
  if (has_database_preprocessing_step()) {
    clear_database_preprocessing_step();
  }
}

void DatabasePreprocessingStep::SetCachedSize(int size) const {
  _impl_._cached_size_.Set(size);
}

void DatabasePreprocessingStep::clear_database_preprocessing_step() {
// @@protoc_insertion_point(one_of_clear_start:second.protos.DatabasePreprocessingStep)
  switch (database_preprocessing_step_case()) {
    case kFilterByDifficulty: {
      if (GetArenaForAllocation() == nullptr) {
        delete _impl_.database_preprocessing_step_.filter_by_difficulty_;
      }
      break;
    }
    case kFilterByMinNumPoints: {
      if (GetArenaForAllocation() == nullptr) {
        delete _impl_.database_preprocessing_step_.filter_by_min_num_points_;
      }
      break;
    }
    case DATABASE_PREPROCESSING_STEP_NOT_SET: {
      break;
    }
  }
  _impl_._oneof_case_[0] = DATABASE_PREPROCESSING_STEP_NOT_SET;
}


void DatabasePreprocessingStep::Clear() {
// @@protoc_insertion_point(message_clear_start:second.protos.DatabasePreprocessingStep)
  uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  clear_database_preprocessing_step();
  _internal_metadata_.Clear<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>();
}

const char* DatabasePreprocessingStep::_InternalParse(const char* ptr, ::_pbi::ParseContext* ctx) {
#define CHK_(x) if (PROTOBUF_PREDICT_FALSE(!(x))) goto failure
  while (!ctx->Done(&ptr)) {
    uint32_t tag;
    ptr = ::_pbi::ReadTag(ptr, &tag);
    switch (tag >> 3) {
      // .second.protos.DBFilterByDifficulty filter_by_difficulty = 1;
      case 1:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 10)) {
          ptr = ctx->ParseMessage(_internal_mutable_filter_by_difficulty(), ptr);
          CHK_(ptr);
        } else
          goto handle_unusual;
        continue;
      // .second.protos.DBFilterByMinNumPointInGroundTruth filter_by_min_num_points = 2;
      case 2:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 18)) {
          ptr = ctx->ParseMessage(_internal_mutable_filter_by_min_num_points(), ptr);
          CHK_(ptr);
        } else
          goto handle_unusual;
        continue;
      default:
        goto handle_unusual;
    }  // switch
  handle_unusual:
    if ((tag == 0) || ((tag & 7) == 4)) {
      CHK_(ptr);
      ctx->SetLastTag(tag);
      goto message_done;
    }
    ptr = UnknownFieldParse(
        tag,
        _internal_metadata_.mutable_unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(),
        ptr, ctx);
    CHK_(ptr != nullptr);
  }  // while
message_done:
  return ptr;
failure:
  ptr = nullptr;
  goto message_done;
#undef CHK_
}

uint8_t* DatabasePreprocessingStep::_InternalSerialize(
    uint8_t* target, ::PROTOBUF_NAMESPACE_ID::io::EpsCopyOutputStream* stream) const {
  // @@protoc_insertion_point(serialize_to_array_start:second.protos.DatabasePreprocessingStep)
  uint32_t cached_has_bits = 0;
  (void) cached_has_bits;

  // .second.protos.DBFilterByDifficulty filter_by_difficulty = 1;
  if (_internal_has_filter_by_difficulty()) {
    target = ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::
      InternalWriteMessage(1, _Internal::filter_by_difficulty(this),
        _Internal::filter_by_difficulty(this).GetCachedSize(), target, stream);
  }

  // .second.protos.DBFilterByMinNumPointInGroundTruth filter_by_min_num_points = 2;
  if (_internal_has_filter_by_min_num_points()) {
    target = ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::
      InternalWriteMessage(2, _Internal::filter_by_min_num_points(this),
        _Internal::filter_by_min_num_points(this).GetCachedSize(), target, stream);
  }

  if (PROTOBUF_PREDICT_FALSE(_internal_metadata_.have_unknown_fields())) {
    target = ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
        _internal_metadata_.unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(::PROTOBUF_NAMESPACE_ID::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:second.protos.DatabasePreprocessingStep)
  return target;
}

size_t DatabasePreprocessingStep::ByteSizeLong() const {
// @@protoc_insertion_point(message_byte_size_start:second.protos.DatabasePreprocessingStep)
  size_t total_size = 0;

  uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  switch (database_preprocessing_step_case()) {
    // .second.protos.DBFilterByDifficulty filter_by_difficulty = 1;
    case kFilterByDifficulty: {
      total_size += 1 +
        ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::MessageSize(
          *_impl_.database_preprocessing_step_.filter_by_difficulty_);
      break;
    }
    // .second.protos.DBFilterByMinNumPointInGroundTruth filter_by_min_num_points = 2;
    case kFilterByMinNumPoints: {
      total_size += 1 +
        ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::MessageSize(
          *_impl_.database_preprocessing_step_.filter_by_min_num_points_);
      break;
    }
    case DATABASE_PREPROCESSING_STEP_NOT_SET: {
      break;
    }
  }
  return MaybeComputeUnknownFieldsSize(total_size, &_impl_._cached_size_);
}

const ::PROTOBUF_NAMESPACE_ID::Message::ClassData DatabasePreprocessingStep::_class_data_ = {
    ::PROTOBUF_NAMESPACE_ID::Message::CopyWithSourceCheck,
    DatabasePreprocessingStep::MergeImpl
};
const ::PROTOBUF_NAMESPACE_ID::Message::ClassData*DatabasePreprocessingStep::GetClassData() const { return &_class_data_; }


void DatabasePreprocessingStep::MergeImpl(::PROTOBUF_NAMESPACE_ID::Message& to_msg, const ::PROTOBUF_NAMESPACE_ID::Message& from_msg) {
  auto* const _this = static_cast<DatabasePreprocessingStep*>(&to_msg);
  auto& from = static_cast<const DatabasePreprocessingStep&>(from_msg);
  // @@protoc_insertion_point(class_specific_merge_from_start:second.protos.DatabasePreprocessingStep)
  GOOGLE_DCHECK_NE(&from, _this);
  uint32_t cached_has_bits = 0;
  (void) cached_has_bits;

  switch (from.database_preprocessing_step_case()) {
    case kFilterByDifficulty: {
      _this->_internal_mutable_filter_by_difficulty()->::second::protos::DBFilterByDifficulty::MergeFrom(
          from._internal_filter_by_difficulty());
      break;
    }
    case kFilterByMinNumPoints: {
      _this->_internal_mutable_filter_by_min_num_points()->::second::protos::DBFilterByMinNumPointInGroundTruth::MergeFrom(
          from._internal_filter_by_min_num_points());
      break;
    }
    case DATABASE_PREPROCESSING_STEP_NOT_SET: {
      break;
    }
  }
  _this->_internal_metadata_.MergeFrom<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(from._internal_metadata_);
}

void DatabasePreprocessingStep::CopyFrom(const DatabasePreprocessingStep& from) {
// @@protoc_insertion_point(class_specific_copy_from_start:second.protos.DatabasePreprocessingStep)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}

bool DatabasePreprocessingStep::IsInitialized() const {
  return true;
}

void DatabasePreprocessingStep::InternalSwap(DatabasePreprocessingStep* other) {
  using std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_.database_preprocessing_step_, other->_impl_.database_preprocessing_step_);
  swap(_impl_._oneof_case_[0], other->_impl_._oneof_case_[0]);
}

::PROTOBUF_NAMESPACE_ID::Metadata DatabasePreprocessingStep::GetMetadata() const {
  return ::_pbi::AssignDescriptors(
      &descriptor_table_second_2fprotos_2fpreprocess_2eproto_getter, &descriptor_table_second_2fprotos_2fpreprocess_2eproto_once,
      file_level_metadata_second_2fprotos_2fpreprocess_2eproto[1]);
}

// ===================================================================

class DBFilterByDifficulty::_Internal {
 public:
};

DBFilterByDifficulty::DBFilterByDifficulty(::PROTOBUF_NAMESPACE_ID::Arena* arena,
                         bool is_message_owned)
  : ::PROTOBUF_NAMESPACE_ID::Message(arena, is_message_owned) {
  SharedCtor(arena, is_message_owned);
  // @@protoc_insertion_point(arena_constructor:second.protos.DBFilterByDifficulty)
}
DBFilterByDifficulty::DBFilterByDifficulty(const DBFilterByDifficulty& from)
  : ::PROTOBUF_NAMESPACE_ID::Message() {
  DBFilterByDifficulty* const _this = this; (void)_this;
  new (&_impl_) Impl_{
      decltype(_impl_.removed_difficulties_){from._impl_.removed_difficulties_}
    , /*decltype(_impl_._removed_difficulties_cached_byte_size_)*/{0}
    , /*decltype(_impl_._cached_size_)*/{}};

  _internal_metadata_.MergeFrom<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(from._internal_metadata_);
  // @@protoc_insertion_point(copy_constructor:second.protos.DBFilterByDifficulty)
}

inline void DBFilterByDifficulty::SharedCtor(
    ::_pb::Arena* arena, bool is_message_owned) {
  (void)arena;
  (void)is_message_owned;
  new (&_impl_) Impl_{
      decltype(_impl_.removed_difficulties_){arena}
    , /*decltype(_impl_._removed_difficulties_cached_byte_size_)*/{0}
    , /*decltype(_impl_._cached_size_)*/{}
  };
}

DBFilterByDifficulty::~DBFilterByDifficulty() {
  // @@protoc_insertion_point(destructor:second.protos.DBFilterByDifficulty)
  if (auto *arena = _internal_metadata_.DeleteReturnArena<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>()) {
  (void)arena;
    return;
  }
  SharedDtor();
}

inline void DBFilterByDifficulty::SharedDtor() {
  GOOGLE_DCHECK(GetArenaForAllocation() == nullptr);
  _impl_.removed_difficulties_.~RepeatedField();
}

void DBFilterByDifficulty::SetCachedSize(int size) const {
  _impl_._cached_size_.Set(size);
}

void DBFilterByDifficulty::Clear() {
// @@protoc_insertion_point(message_clear_start:second.protos.DBFilterByDifficulty)
  uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  _impl_.removed_difficulties_.Clear();
  _internal_metadata_.Clear<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>();
}

const char* DBFilterByDifficulty::_InternalParse(const char* ptr, ::_pbi::ParseContext* ctx) {
#define CHK_(x) if (PROTOBUF_PREDICT_FALSE(!(x))) goto failure
  while (!ctx->Done(&ptr)) {
    uint32_t tag;
    ptr = ::_pbi::ReadTag(ptr, &tag);
    switch (tag >> 3) {
      // repeated int32 removed_difficulties = 1;
      case 1:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 10)) {
          ptr = ::PROTOBUF_NAMESPACE_ID::internal::PackedInt32Parser(_internal_mutable_removed_difficulties(), ptr, ctx);
          CHK_(ptr);
        } else if (static_cast<uint8_t>(tag) == 8) {
          _internal_add_removed_difficulties(::PROTOBUF_NAMESPACE_ID::internal::ReadVarint32(&ptr));
          CHK_(ptr);
        } else
          goto handle_unusual;
        continue;
      default:
        goto handle_unusual;
    }  // switch
  handle_unusual:
    if ((tag == 0) || ((tag & 7) == 4)) {
      CHK_(ptr);
      ctx->SetLastTag(tag);
      goto message_done;
    }
    ptr = UnknownFieldParse(
        tag,
        _internal_metadata_.mutable_unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(),
        ptr, ctx);
    CHK_(ptr != nullptr);
  }  // while
message_done:
  return ptr;
failure:
  ptr = nullptr;
  goto message_done;
#undef CHK_
}

uint8_t* DBFilterByDifficulty::_InternalSerialize(
    uint8_t* target, ::PROTOBUF_NAMESPACE_ID::io::EpsCopyOutputStream* stream) const {
  // @@protoc_insertion_point(serialize_to_array_start:second.protos.DBFilterByDifficulty)
  uint32_t cached_has_bits = 0;
  (void) cached_has_bits;

  // repeated int32 removed_difficulties = 1;
  {
    int byte_size = _impl_._removed_difficulties_cached_byte_size_.load(std::memory_order_relaxed);
    if (byte_size > 0) {
      target = stream->WriteInt32Packed(
          1, _internal_removed_difficulties(), byte_size, target);
    }
  }

  if (PROTOBUF_PREDICT_FALSE(_internal_metadata_.have_unknown_fields())) {
    target = ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
        _internal_metadata_.unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(::PROTOBUF_NAMESPACE_ID::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:second.protos.DBFilterByDifficulty)
  return target;
}

size_t DBFilterByDifficulty::ByteSizeLong() const {
// @@protoc_insertion_point(message_byte_size_start:second.protos.DBFilterByDifficulty)
  size_t total_size = 0;

  uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  // repeated int32 removed_difficulties = 1;
  {
    size_t data_size = ::_pbi::WireFormatLite::
      Int32Size(this->_impl_.removed_difficulties_);
    if (data_size > 0) {
      total_size += 1 +
        ::_pbi::WireFormatLite::Int32Size(static_cast<int32_t>(data_size));
    }
    int cached_size = ::_pbi::ToCachedSize(data_size);
    _impl_._removed_difficulties_cached_byte_size_.store(cached_size,
                                    std::memory_order_relaxed);
    total_size += data_size;
  }

  return MaybeComputeUnknownFieldsSize(total_size, &_impl_._cached_size_);
}

const ::PROTOBUF_NAMESPACE_ID::Message::ClassData DBFilterByDifficulty::_class_data_ = {
    ::PROTOBUF_NAMESPACE_ID::Message::CopyWithSourceCheck,
    DBFilterByDifficulty::MergeImpl
};
const ::PROTOBUF_NAMESPACE_ID::Message::ClassData*DBFilterByDifficulty::GetClassData() const { return &_class_data_; }


void DBFilterByDifficulty::MergeImpl(::PROTOBUF_NAMESPACE_ID::Message& to_msg, const ::PROTOBUF_NAMESPACE_ID::Message& from_msg) {
  auto* const _this = static_cast<DBFilterByDifficulty*>(&to_msg);
  auto& from = static_cast<const DBFilterByDifficulty&>(from_msg);
  // @@protoc_insertion_point(class_specific_merge_from_start:second.protos.DBFilterByDifficulty)
  GOOGLE_DCHECK_NE(&from, _this);
  uint32_t cached_has_bits = 0;
  (void) cached_has_bits;

  _this->_impl_.removed_difficulties_.MergeFrom(from._impl_.removed_difficulties_);
  _this->_internal_metadata_.MergeFrom<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(from._internal_metadata_);
}

void DBFilterByDifficulty::CopyFrom(const DBFilterByDifficulty& from) {
// @@protoc_insertion_point(class_specific_copy_from_start:second.protos.DBFilterByDifficulty)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}

bool DBFilterByDifficulty::IsInitialized() const {
  return true;
}

void DBFilterByDifficulty::InternalSwap(DBFilterByDifficulty* other) {
  using std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  _impl_.removed_difficulties_.InternalSwap(&other->_impl_.removed_difficulties_);
}

::PROTOBUF_NAMESPACE_ID::Metadata DBFilterByDifficulty::GetMetadata() const {
  return ::_pbi::AssignDescriptors(
      &descriptor_table_second_2fprotos_2fpreprocess_2eproto_getter, &descriptor_table_second_2fprotos_2fpreprocess_2eproto_once,
      file_level_metadata_second_2fprotos_2fpreprocess_2eproto[2]);
}

// ===================================================================

DBFilterByMinNumPointInGroundTruth_MinNumPointPairsEntry_DoNotUse::DBFilterByMinNumPointInGroundTruth_MinNumPointPairsEntry_DoNotUse() {}
DBFilterByMinNumPointInGroundTruth_MinNumPointPairsEntry_DoNotUse::DBFilterByMinNumPointInGroundTruth_MinNumPointPairsEntry_DoNotUse(::PROTOBUF_NAMESPACE_ID::Arena* arena)
    : SuperType(arena) {}
void DBFilterByMinNumPointInGroundTruth_MinNumPointPairsEntry_DoNotUse::MergeFrom(const DBFilterByMinNumPointInGroundTruth_MinNumPointPairsEntry_DoNotUse& other) {
  MergeFromInternal(other);
}
::PROTOBUF_NAMESPACE_ID::Metadata DBFilterByMinNumPointInGroundTruth_MinNumPointPairsEntry_DoNotUse::GetMetadata() const {
  return ::_pbi::AssignDescriptors(
      &descriptor_table_second_2fprotos_2fpreprocess_2eproto_getter, &descriptor_table_second_2fprotos_2fpreprocess_2eproto_once,
      file_level_metadata_second_2fprotos_2fpreprocess_2eproto[3]);
}

// ===================================================================

class DBFilterByMinNumPointInGroundTruth::_Internal {
 public:
};

DBFilterByMinNumPointInGroundTruth::DBFilterByMinNumPointInGroundTruth(::PROTOBUF_NAMESPACE_ID::Arena* arena,
                         bool is_message_owned)
  : ::PROTOBUF_NAMESPACE_ID::Message(arena, is_message_owned) {
  SharedCtor(arena, is_message_owned);
  if (arena != nullptr && !is_message_owned) {
    arena->OwnCustomDestructor(this, &DBFilterByMinNumPointInGroundTruth::ArenaDtor);
  }
  // @@protoc_insertion_point(arena_constructor:second.protos.DBFilterByMinNumPointInGroundTruth)
}
DBFilterByMinNumPointInGroundTruth::DBFilterByMinNumPointInGroundTruth(const DBFilterByMinNumPointInGroundTruth& from)
  : ::PROTOBUF_NAMESPACE_ID::Message() {
  DBFilterByMinNumPointInGroundTruth* const _this = this; (void)_this;
  new (&_impl_) Impl_{
      /*decltype(_impl_.min_num_point_pairs_)*/{}
    , /*decltype(_impl_._cached_size_)*/{}};

  _internal_metadata_.MergeFrom<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(from._internal_metadata_);
  _this->_impl_.min_num_point_pairs_.MergeFrom(from._impl_.min_num_point_pairs_);
  // @@protoc_insertion_point(copy_constructor:second.protos.DBFilterByMinNumPointInGroundTruth)
}

inline void DBFilterByMinNumPointInGroundTruth::SharedCtor(
    ::_pb::Arena* arena, bool is_message_owned) {
  (void)arena;
  (void)is_message_owned;
  new (&_impl_) Impl_{
      /*decltype(_impl_.min_num_point_pairs_)*/{::_pbi::ArenaInitialized(), arena}
    , /*decltype(_impl_._cached_size_)*/{}
  };
}

DBFilterByMinNumPointInGroundTruth::~DBFilterByMinNumPointInGroundTruth() {
  // @@protoc_insertion_point(destructor:second.protos.DBFilterByMinNumPointInGroundTruth)
  if (auto *arena = _internal_metadata_.DeleteReturnArena<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>()) {
  (void)arena;
    ArenaDtor(this);
    return;
  }
  SharedDtor();
}

inline void DBFilterByMinNumPointInGroundTruth::SharedDtor() {
  GOOGLE_DCHECK(GetArenaForAllocation() == nullptr);
  _impl_.min_num_point_pairs_.Destruct();
  _impl_.min_num_point_pairs_.~MapField();
}

void DBFilterByMinNumPointInGroundTruth::ArenaDtor(void* object) {
  DBFilterByMinNumPointInGroundTruth* _this = reinterpret_cast< DBFilterByMinNumPointInGroundTruth* >(object);
  _this->_impl_.min_num_point_pairs_.Destruct();
}
void DBFilterByMinNumPointInGroundTruth::SetCachedSize(int size) const {
  _impl_._cached_size_.Set(size);
}

void DBFilterByMinNumPointInGroundTruth::Clear() {
// @@protoc_insertion_point(message_clear_start:second.protos.DBFilterByMinNumPointInGroundTruth)
  uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  _impl_.min_num_point_pairs_.Clear();
  _internal_metadata_.Clear<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>();
}

const char* DBFilterByMinNumPointInGroundTruth::_InternalParse(const char* ptr, ::_pbi::ParseContext* ctx) {
#define CHK_(x) if (PROTOBUF_PREDICT_FALSE(!(x))) goto failure
  while (!ctx->Done(&ptr)) {
    uint32_t tag;
    ptr = ::_pbi::ReadTag(ptr, &tag);
    switch (tag >> 3) {
      // map<string, uint32> min_num_point_pairs = 1;
      case 1:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 10)) {
          ptr -= 1;
          do {
            ptr += 1;
            ptr = ctx->ParseMessage(&_impl_.min_num_point_pairs_, ptr);
            CHK_(ptr);
            if (!ctx->DataAvailable(ptr)) break;
          } while (::PROTOBUF_NAMESPACE_ID::internal::ExpectTag<10>(ptr));
        } else
          goto handle_unusual;
        continue;
      default:
        goto handle_unusual;
    }  // switch
  handle_unusual:
    if ((tag == 0) || ((tag & 7) == 4)) {
      CHK_(ptr);
      ctx->SetLastTag(tag);
      goto message_done;
    }
    ptr = UnknownFieldParse(
        tag,
        _internal_metadata_.mutable_unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(),
        ptr, ctx);
    CHK_(ptr != nullptr);
  }  // while
message_done:
  return ptr;
failure:
  ptr = nullptr;
  goto message_done;
#undef CHK_
}

uint8_t* DBFilterByMinNumPointInGroundTruth::_InternalSerialize(
    uint8_t* target, ::PROTOBUF_NAMESPACE_ID::io::EpsCopyOutputStream* stream) const {
  // @@protoc_insertion_point(serialize_to_array_start:second.protos.DBFilterByMinNumPointInGroundTruth)
  uint32_t cached_has_bits = 0;
  (void) cached_has_bits;

  // map<string, uint32> min_num_point_pairs = 1;
  if (!this->_internal_min_num_point_pairs().empty()) {
    using MapType = ::_pb::Map<std::string, uint32_t>;
    using WireHelper = DBFilterByMinNumPointInGroundTruth_MinNumPointPairsEntry_DoNotUse::Funcs;
    const auto& map_field = this->_internal_min_num_point_pairs();
    auto check_utf8 = [](const MapType::value_type& entry) {
      (void)entry;
      ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::VerifyUtf8String(
        entry.first.data(), static_cast<int>(entry.first.length()),
        ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::SERIALIZE,
        "second.protos.DBFilterByMinNumPointInGroundTruth.MinNumPointPairsEntry.key");
    };

    if (stream->IsSerializationDeterministic() && map_field.size() > 1) {
      for (const auto& entry : ::_pbi::MapSorterPtr<MapType>(map_field)) {
        target = WireHelper::InternalSerialize(1, entry.first, entry.second, target, stream);
        check_utf8(entry);
      }
    } else {
      for (const auto& entry : map_field) {
        target = WireHelper::InternalSerialize(1, entry.first, entry.second, target, stream);
        check_utf8(entry);
      }
    }
  }

  if (PROTOBUF_PREDICT_FALSE(_internal_metadata_.have_unknown_fields())) {
    target = ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
        _internal_metadata_.unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(::PROTOBUF_NAMESPACE_ID::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:second.protos.DBFilterByMinNumPointInGroundTruth)
  return target;
}

size_t DBFilterByMinNumPointInGroundTruth::ByteSizeLong() const {
// @@protoc_insertion_point(message_byte_size_start:second.protos.DBFilterByMinNumPointInGroundTruth)
  size_t total_size = 0;

  uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  // map<string, uint32> min_num_point_pairs = 1;
  total_size += 1 *
      ::PROTOBUF_NAMESPACE_ID::internal::FromIntSize(this->_internal_min_num_point_pairs_size());
  for (::PROTOBUF_NAMESPACE_ID::Map< std::string, uint32_t >::const_iterator
      it = this->_internal_min_num_point_pairs().begin();
      it != this->_internal_min_num_point_pairs().end(); ++it) {
    total_size += DBFilterByMinNumPointInGroundTruth_MinNumPointPairsEntry_DoNotUse::Funcs::ByteSizeLong(it->first, it->second);
  }

  return MaybeComputeUnknownFieldsSize(total_size, &_impl_._cached_size_);
}

const ::PROTOBUF_NAMESPACE_ID::Message::ClassData DBFilterByMinNumPointInGroundTruth::_class_data_ = {
    ::PROTOBUF_NAMESPACE_ID::Message::CopyWithSourceCheck,
    DBFilterByMinNumPointInGroundTruth::MergeImpl
};
const ::PROTOBUF_NAMESPACE_ID::Message::ClassData*DBFilterByMinNumPointInGroundTruth::GetClassData() const { return &_class_data_; }


void DBFilterByMinNumPointInGroundTruth::MergeImpl(::PROTOBUF_NAMESPACE_ID::Message& to_msg, const ::PROTOBUF_NAMESPACE_ID::Message& from_msg) {
  auto* const _this = static_cast<DBFilterByMinNumPointInGroundTruth*>(&to_msg);
  auto& from = static_cast<const DBFilterByMinNumPointInGroundTruth&>(from_msg);
  // @@protoc_insertion_point(class_specific_merge_from_start:second.protos.DBFilterByMinNumPointInGroundTruth)
  GOOGLE_DCHECK_NE(&from, _this);
  uint32_t cached_has_bits = 0;
  (void) cached_has_bits;

  _this->_impl_.min_num_point_pairs_.MergeFrom(from._impl_.min_num_point_pairs_);
  _this->_internal_metadata_.MergeFrom<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(from._internal_metadata_);
}

void DBFilterByMinNumPointInGroundTruth::CopyFrom(const DBFilterByMinNumPointInGroundTruth& from) {
// @@protoc_insertion_point(class_specific_copy_from_start:second.protos.DBFilterByMinNumPointInGroundTruth)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}

bool DBFilterByMinNumPointInGroundTruth::IsInitialized() const {
  return true;
}

void DBFilterByMinNumPointInGroundTruth::InternalSwap(DBFilterByMinNumPointInGroundTruth* other) {
  using std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  _impl_.min_num_point_pairs_.InternalSwap(&other->_impl_.min_num_point_pairs_);
}

::PROTOBUF_NAMESPACE_ID::Metadata DBFilterByMinNumPointInGroundTruth::GetMetadata() const {
  return ::_pbi::AssignDescriptors(
      &descriptor_table_second_2fprotos_2fpreprocess_2eproto_getter, &descriptor_table_second_2fprotos_2fpreprocess_2eproto_once,
      file_level_metadata_second_2fprotos_2fpreprocess_2eproto[4]);
}

// ===================================================================

class PreprocessingStep::_Internal {
 public:
  static const ::second::protos::RandomGlobalScaling& random_global_scaling(const PreprocessingStep* msg);
  static const ::second::protos::RandomGlobalRotation& random_global_rotation(const PreprocessingStep* msg);
};

const ::second::protos::RandomGlobalScaling&
PreprocessingStep::_Internal::random_global_scaling(const PreprocessingStep* msg) {
  return *msg->_impl_.preprocessing_step_.random_global_scaling_;
}
const ::second::protos::RandomGlobalRotation&
PreprocessingStep::_Internal::random_global_rotation(const PreprocessingStep* msg) {
  return *msg->_impl_.preprocessing_step_.random_global_rotation_;
}
void PreprocessingStep::set_allocated_random_global_scaling(::second::protos::RandomGlobalScaling* random_global_scaling) {
  ::PROTOBUF_NAMESPACE_ID::Arena* message_arena = GetArenaForAllocation();
  clear_preprocessing_step();
  if (random_global_scaling) {
    ::PROTOBUF_NAMESPACE_ID::Arena* submessage_arena =
      ::PROTOBUF_NAMESPACE_ID::Arena::InternalGetOwningArena(random_global_scaling);
    if (message_arena != submessage_arena) {
      random_global_scaling = ::PROTOBUF_NAMESPACE_ID::internal::GetOwnedMessage(
          message_arena, random_global_scaling, submessage_arena);
    }
    set_has_random_global_scaling();
    _impl_.preprocessing_step_.random_global_scaling_ = random_global_scaling;
  }
  // @@protoc_insertion_point(field_set_allocated:second.protos.PreprocessingStep.random_global_scaling)
}
void PreprocessingStep::set_allocated_random_global_rotation(::second::protos::RandomGlobalRotation* random_global_rotation) {
  ::PROTOBUF_NAMESPACE_ID::Arena* message_arena = GetArenaForAllocation();
  clear_preprocessing_step();
  if (random_global_rotation) {
    ::PROTOBUF_NAMESPACE_ID::Arena* submessage_arena =
      ::PROTOBUF_NAMESPACE_ID::Arena::InternalGetOwningArena(random_global_rotation);
    if (message_arena != submessage_arena) {
      random_global_rotation = ::PROTOBUF_NAMESPACE_ID::internal::GetOwnedMessage(
          message_arena, random_global_rotation, submessage_arena);
    }
    set_has_random_global_rotation();
    _impl_.preprocessing_step_.random_global_rotation_ = random_global_rotation;
  }
  // @@protoc_insertion_point(field_set_allocated:second.protos.PreprocessingStep.random_global_rotation)
}
PreprocessingStep::PreprocessingStep(::PROTOBUF_NAMESPACE_ID::Arena* arena,
                         bool is_message_owned)
  : ::PROTOBUF_NAMESPACE_ID::Message(arena, is_message_owned) {
  SharedCtor(arena, is_message_owned);
  // @@protoc_insertion_point(arena_constructor:second.protos.PreprocessingStep)
}
PreprocessingStep::PreprocessingStep(const PreprocessingStep& from)
  : ::PROTOBUF_NAMESPACE_ID::Message() {
  PreprocessingStep* const _this = this; (void)_this;
  new (&_impl_) Impl_{
      decltype(_impl_.preprocessing_step_){}
    , /*decltype(_impl_._cached_size_)*/{}
    , /*decltype(_impl_._oneof_case_)*/{}};

  _internal_metadata_.MergeFrom<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(from._internal_metadata_);
  clear_has_preprocessing_step();
  switch (from.preprocessing_step_case()) {
    case kRandomGlobalScaling: {
      _this->_internal_mutable_random_global_scaling()->::second::protos::RandomGlobalScaling::MergeFrom(
          from._internal_random_global_scaling());
      break;
    }
    case kRandomGlobalRotation: {
      _this->_internal_mutable_random_global_rotation()->::second::protos::RandomGlobalRotation::MergeFrom(
          from._internal_random_global_rotation());
      break;
    }
    case PREPROCESSING_STEP_NOT_SET: {
      break;
    }
  }
  // @@protoc_insertion_point(copy_constructor:second.protos.PreprocessingStep)
}

inline void PreprocessingStep::SharedCtor(
    ::_pb::Arena* arena, bool is_message_owned) {
  (void)arena;
  (void)is_message_owned;
  new (&_impl_) Impl_{
      decltype(_impl_.preprocessing_step_){}
    , /*decltype(_impl_._cached_size_)*/{}
    , /*decltype(_impl_._oneof_case_)*/{}
  };
  clear_has_preprocessing_step();
}

PreprocessingStep::~PreprocessingStep() {
  // @@protoc_insertion_point(destructor:second.protos.PreprocessingStep)
  if (auto *arena = _internal_metadata_.DeleteReturnArena<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>()) {
  (void)arena;
    return;
  }
  SharedDtor();
}

inline void PreprocessingStep::SharedDtor() {
  GOOGLE_DCHECK(GetArenaForAllocation() == nullptr);
  if (has_preprocessing_step()) {
    clear_preprocessing_step();
  }
}

void PreprocessingStep::SetCachedSize(int size) const {
  _impl_._cached_size_.Set(size);
}

void PreprocessingStep::clear_preprocessing_step() {
// @@protoc_insertion_point(one_of_clear_start:second.protos.PreprocessingStep)
  switch (preprocessing_step_case()) {
    case kRandomGlobalScaling: {
      if (GetArenaForAllocation() == nullptr) {
        delete _impl_.preprocessing_step_.random_global_scaling_;
      }
      break;
    }
    case kRandomGlobalRotation: {
      if (GetArenaForAllocation() == nullptr) {
        delete _impl_.preprocessing_step_.random_global_rotation_;
      }
      break;
    }
    case PREPROCESSING_STEP_NOT_SET: {
      break;
    }
  }
  _impl_._oneof_case_[0] = PREPROCESSING_STEP_NOT_SET;
}


void PreprocessingStep::Clear() {
// @@protoc_insertion_point(message_clear_start:second.protos.PreprocessingStep)
  uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  clear_preprocessing_step();
  _internal_metadata_.Clear<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>();
}

const char* PreprocessingStep::_InternalParse(const char* ptr, ::_pbi::ParseContext* ctx) {
#define CHK_(x) if (PROTOBUF_PREDICT_FALSE(!(x))) goto failure
  while (!ctx->Done(&ptr)) {
    uint32_t tag;
    ptr = ::_pbi::ReadTag(ptr, &tag);
    switch (tag >> 3) {
      // .second.protos.RandomGlobalScaling random_global_scaling = 1;
      case 1:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 10)) {
          ptr = ctx->ParseMessage(_internal_mutable_random_global_scaling(), ptr);
          CHK_(ptr);
        } else
          goto handle_unusual;
        continue;
      // .second.protos.RandomGlobalRotation random_global_rotation = 2;
      case 2:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 18)) {
          ptr = ctx->ParseMessage(_internal_mutable_random_global_rotation(), ptr);
          CHK_(ptr);
        } else
          goto handle_unusual;
        continue;
      default:
        goto handle_unusual;
    }  // switch
  handle_unusual:
    if ((tag == 0) || ((tag & 7) == 4)) {
      CHK_(ptr);
      ctx->SetLastTag(tag);
      goto message_done;
    }
    ptr = UnknownFieldParse(
        tag,
        _internal_metadata_.mutable_unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(),
        ptr, ctx);
    CHK_(ptr != nullptr);
  }  // while
message_done:
  return ptr;
failure:
  ptr = nullptr;
  goto message_done;
#undef CHK_
}

uint8_t* PreprocessingStep::_InternalSerialize(
    uint8_t* target, ::PROTOBUF_NAMESPACE_ID::io::EpsCopyOutputStream* stream) const {
  // @@protoc_insertion_point(serialize_to_array_start:second.protos.PreprocessingStep)
  uint32_t cached_has_bits = 0;
  (void) cached_has_bits;

  // .second.protos.RandomGlobalScaling random_global_scaling = 1;
  if (_internal_has_random_global_scaling()) {
    target = ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::
      InternalWriteMessage(1, _Internal::random_global_scaling(this),
        _Internal::random_global_scaling(this).GetCachedSize(), target, stream);
  }

  // .second.protos.RandomGlobalRotation random_global_rotation = 2;
  if (_internal_has_random_global_rotation()) {
    target = ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::
      InternalWriteMessage(2, _Internal::random_global_rotation(this),
        _Internal::random_global_rotation(this).GetCachedSize(), target, stream);
  }

  if (PROTOBUF_PREDICT_FALSE(_internal_metadata_.have_unknown_fields())) {
    target = ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
        _internal_metadata_.unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(::PROTOBUF_NAMESPACE_ID::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:second.protos.PreprocessingStep)
  return target;
}

size_t PreprocessingStep::ByteSizeLong() const {
// @@protoc_insertion_point(message_byte_size_start:second.protos.PreprocessingStep)
  size_t total_size = 0;

  uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  switch (preprocessing_step_case()) {
    // .second.protos.RandomGlobalScaling random_global_scaling = 1;
    case kRandomGlobalScaling: {
      total_size += 1 +
        ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::MessageSize(
          *_impl_.preprocessing_step_.random_global_scaling_);
      break;
    }
    // .second.protos.RandomGlobalRotation random_global_rotation = 2;
    case kRandomGlobalRotation: {
      total_size += 1 +
        ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::MessageSize(
          *_impl_.preprocessing_step_.random_global_rotation_);
      break;
    }
    case PREPROCESSING_STEP_NOT_SET: {
      break;
    }
  }
  return MaybeComputeUnknownFieldsSize(total_size, &_impl_._cached_size_);
}

const ::PROTOBUF_NAMESPACE_ID::Message::ClassData PreprocessingStep::_class_data_ = {
    ::PROTOBUF_NAMESPACE_ID::Message::CopyWithSourceCheck,
    PreprocessingStep::MergeImpl
};
const ::PROTOBUF_NAMESPACE_ID::Message::ClassData*PreprocessingStep::GetClassData() const { return &_class_data_; }


void PreprocessingStep::MergeImpl(::PROTOBUF_NAMESPACE_ID::Message& to_msg, const ::PROTOBUF_NAMESPACE_ID::Message& from_msg) {
  auto* const _this = static_cast<PreprocessingStep*>(&to_msg);
  auto& from = static_cast<const PreprocessingStep&>(from_msg);
  // @@protoc_insertion_point(class_specific_merge_from_start:second.protos.PreprocessingStep)
  GOOGLE_DCHECK_NE(&from, _this);
  uint32_t cached_has_bits = 0;
  (void) cached_has_bits;

  switch (from.preprocessing_step_case()) {
    case kRandomGlobalScaling: {
      _this->_internal_mutable_random_global_scaling()->::second::protos::RandomGlobalScaling::MergeFrom(
          from._internal_random_global_scaling());
      break;
    }
    case kRandomGlobalRotation: {
      _this->_internal_mutable_random_global_rotation()->::second::protos::RandomGlobalRotation::MergeFrom(
          from._internal_random_global_rotation());
      break;
    }
    case PREPROCESSING_STEP_NOT_SET: {
      break;
    }
  }
  _this->_internal_metadata_.MergeFrom<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(from._internal_metadata_);
}

void PreprocessingStep::CopyFrom(const PreprocessingStep& from) {
// @@protoc_insertion_point(class_specific_copy_from_start:second.protos.PreprocessingStep)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}

bool PreprocessingStep::IsInitialized() const {
  return true;
}

void PreprocessingStep::InternalSwap(PreprocessingStep* other) {
  using std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_.preprocessing_step_, other->_impl_.preprocessing_step_);
  swap(_impl_._oneof_case_[0], other->_impl_._oneof_case_[0]);
}

::PROTOBUF_NAMESPACE_ID::Metadata PreprocessingStep::GetMetadata() const {
  return ::_pbi::AssignDescriptors(
      &descriptor_table_second_2fprotos_2fpreprocess_2eproto_getter, &descriptor_table_second_2fprotos_2fpreprocess_2eproto_once,
      file_level_metadata_second_2fprotos_2fpreprocess_2eproto[5]);
}

// ===================================================================

class RandomGlobalScaling::_Internal {
 public:
};

RandomGlobalScaling::RandomGlobalScaling(::PROTOBUF_NAMESPACE_ID::Arena* arena,
                         bool is_message_owned)
  : ::PROTOBUF_NAMESPACE_ID::Message(arena, is_message_owned) {
  SharedCtor(arena, is_message_owned);
  // @@protoc_insertion_point(arena_constructor:second.protos.RandomGlobalScaling)
}
RandomGlobalScaling::RandomGlobalScaling(const RandomGlobalScaling& from)
  : ::PROTOBUF_NAMESPACE_ID::Message() {
  RandomGlobalScaling* const _this = this; (void)_this;
  new (&_impl_) Impl_{
      decltype(_impl_.min_scale_){}
    , decltype(_impl_.max_scale_){}
    , /*decltype(_impl_._cached_size_)*/{}};

  _internal_metadata_.MergeFrom<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(from._internal_metadata_);
  ::memcpy(&_impl_.min_scale_, &from._impl_.min_scale_,
    static_cast<size_t>(reinterpret_cast<char*>(&_impl_.max_scale_) -
    reinterpret_cast<char*>(&_impl_.min_scale_)) + sizeof(_impl_.max_scale_));
  // @@protoc_insertion_point(copy_constructor:second.protos.RandomGlobalScaling)
}

inline void RandomGlobalScaling::SharedCtor(
    ::_pb::Arena* arena, bool is_message_owned) {
  (void)arena;
  (void)is_message_owned;
  new (&_impl_) Impl_{
      decltype(_impl_.min_scale_){0}
    , decltype(_impl_.max_scale_){0}
    , /*decltype(_impl_._cached_size_)*/{}
  };
}

RandomGlobalScaling::~RandomGlobalScaling() {
  // @@protoc_insertion_point(destructor:second.protos.RandomGlobalScaling)
  if (auto *arena = _internal_metadata_.DeleteReturnArena<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>()) {
  (void)arena;
    return;
  }
  SharedDtor();
}

inline void RandomGlobalScaling::SharedDtor() {
  GOOGLE_DCHECK(GetArenaForAllocation() == nullptr);
}

void RandomGlobalScaling::SetCachedSize(int size) const {
  _impl_._cached_size_.Set(size);
}

void RandomGlobalScaling::Clear() {
// @@protoc_insertion_point(message_clear_start:second.protos.RandomGlobalScaling)
  uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  ::memset(&_impl_.min_scale_, 0, static_cast<size_t>(
      reinterpret_cast<char*>(&_impl_.max_scale_) -
      reinterpret_cast<char*>(&_impl_.min_scale_)) + sizeof(_impl_.max_scale_));
  _internal_metadata_.Clear<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>();
}

const char* RandomGlobalScaling::_InternalParse(const char* ptr, ::_pbi::ParseContext* ctx) {
#define CHK_(x) if (PROTOBUF_PREDICT_FALSE(!(x))) goto failure
  while (!ctx->Done(&ptr)) {
    uint32_t tag;
    ptr = ::_pbi::ReadTag(ptr, &tag);
    switch (tag >> 3) {
      // float min_scale = 1;
      case 1:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 13)) {
          _impl_.min_scale_ = ::PROTOBUF_NAMESPACE_ID::internal::UnalignedLoad<float>(ptr);
          ptr += sizeof(float);
        } else
          goto handle_unusual;
        continue;
      // float max_scale = 2;
      case 2:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 21)) {
          _impl_.max_scale_ = ::PROTOBUF_NAMESPACE_ID::internal::UnalignedLoad<float>(ptr);
          ptr += sizeof(float);
        } else
          goto handle_unusual;
        continue;
      default:
        goto handle_unusual;
    }  // switch
  handle_unusual:
    if ((tag == 0) || ((tag & 7) == 4)) {
      CHK_(ptr);
      ctx->SetLastTag(tag);
      goto message_done;
    }
    ptr = UnknownFieldParse(
        tag,
        _internal_metadata_.mutable_unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(),
        ptr, ctx);
    CHK_(ptr != nullptr);
  }  // while
message_done:
  return ptr;
failure:
  ptr = nullptr;
  goto message_done;
#undef CHK_
}

uint8_t* RandomGlobalScaling::_InternalSerialize(
    uint8_t* target, ::PROTOBUF_NAMESPACE_ID::io::EpsCopyOutputStream* stream) const {
  // @@protoc_insertion_point(serialize_to_array_start:second.protos.RandomGlobalScaling)
  uint32_t cached_has_bits = 0;
  (void) cached_has_bits;

  // float min_scale = 1;
  static_assert(sizeof(uint32_t) == sizeof(float), "Code assumes uint32_t and float are the same size.");
  float tmp_min_scale = this->_internal_min_scale();
  uint32_t raw_min_scale;
  memcpy(&raw_min_scale, &tmp_min_scale, sizeof(tmp_min_scale));
  if (raw_min_scale != 0) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteFloatToArray(1, this->_internal_min_scale(), target);
  }

  // float max_scale = 2;
  static_assert(sizeof(uint32_t) == sizeof(float), "Code assumes uint32_t and float are the same size.");
  float tmp_max_scale = this->_internal_max_scale();
  uint32_t raw_max_scale;
  memcpy(&raw_max_scale, &tmp_max_scale, sizeof(tmp_max_scale));
  if (raw_max_scale != 0) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteFloatToArray(2, this->_internal_max_scale(), target);
  }

  if (PROTOBUF_PREDICT_FALSE(_internal_metadata_.have_unknown_fields())) {
    target = ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
        _internal_metadata_.unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(::PROTOBUF_NAMESPACE_ID::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:second.protos.RandomGlobalScaling)
  return target;
}

size_t RandomGlobalScaling::ByteSizeLong() const {
// @@protoc_insertion_point(message_byte_size_start:second.protos.RandomGlobalScaling)
  size_t total_size = 0;

  uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  // float min_scale = 1;
  static_assert(sizeof(uint32_t) == sizeof(float), "Code assumes uint32_t and float are the same size.");
  float tmp_min_scale = this->_internal_min_scale();
  uint32_t raw_min_scale;
  memcpy(&raw_min_scale, &tmp_min_scale, sizeof(tmp_min_scale));
  if (raw_min_scale != 0) {
    total_size += 1 + 4;
  }

  // float max_scale = 2;
  static_assert(sizeof(uint32_t) == sizeof(float), "Code assumes uint32_t and float are the same size.");
  float tmp_max_scale = this->_internal_max_scale();
  uint32_t raw_max_scale;
  memcpy(&raw_max_scale, &tmp_max_scale, sizeof(tmp_max_scale));
  if (raw_max_scale != 0) {
    total_size += 1 + 4;
  }

  return MaybeComputeUnknownFieldsSize(total_size, &_impl_._cached_size_);
}

const ::PROTOBUF_NAMESPACE_ID::Message::ClassData RandomGlobalScaling::_class_data_ = {
    ::PROTOBUF_NAMESPACE_ID::Message::CopyWithSourceCheck,
    RandomGlobalScaling::MergeImpl
};
const ::PROTOBUF_NAMESPACE_ID::Message::ClassData*RandomGlobalScaling::GetClassData() const { return &_class_data_; }


void RandomGlobalScaling::MergeImpl(::PROTOBUF_NAMESPACE_ID::Message& to_msg, const ::PROTOBUF_NAMESPACE_ID::Message& from_msg) {
  auto* const _this = static_cast<RandomGlobalScaling*>(&to_msg);
  auto& from = static_cast<const RandomGlobalScaling&>(from_msg);
  // @@protoc_insertion_point(class_specific_merge_from_start:second.protos.RandomGlobalScaling)
  GOOGLE_DCHECK_NE(&from, _this);
  uint32_t cached_has_bits = 0;
  (void) cached_has_bits;

  static_assert(sizeof(uint32_t) == sizeof(float), "Code assumes uint32_t and float are the same size.");
  float tmp_min_scale = from._internal_min_scale();
  uint32_t raw_min_scale;
  memcpy(&raw_min_scale, &tmp_min_scale, sizeof(tmp_min_scale));
  if (raw_min_scale != 0) {
    _this->_internal_set_min_scale(from._internal_min_scale());
  }
  static_assert(sizeof(uint32_t) == sizeof(float), "Code assumes uint32_t and float are the same size.");
  float tmp_max_scale = from._internal_max_scale();
  uint32_t raw_max_scale;
  memcpy(&raw_max_scale, &tmp_max_scale, sizeof(tmp_max_scale));
  if (raw_max_scale != 0) {
    _this->_internal_set_max_scale(from._internal_max_scale());
  }
  _this->_internal_metadata_.MergeFrom<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(from._internal_metadata_);
}

void RandomGlobalScaling::CopyFrom(const RandomGlobalScaling& from) {
// @@protoc_insertion_point(class_specific_copy_from_start:second.protos.RandomGlobalScaling)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}

bool RandomGlobalScaling::IsInitialized() const {
  return true;
}

void RandomGlobalScaling::InternalSwap(RandomGlobalScaling* other) {
  using std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  ::PROTOBUF_NAMESPACE_ID::internal::memswap<
      PROTOBUF_FIELD_OFFSET(RandomGlobalScaling, _impl_.max_scale_)
      + sizeof(RandomGlobalScaling::_impl_.max_scale_)
      - PROTOBUF_FIELD_OFFSET(RandomGlobalScaling, _impl_.min_scale_)>(
          reinterpret_cast<char*>(&_impl_.min_scale_),
          reinterpret_cast<char*>(&other->_impl_.min_scale_));
}

::PROTOBUF_NAMESPACE_ID::Metadata RandomGlobalScaling::GetMetadata() const {
  return ::_pbi::AssignDescriptors(
      &descriptor_table_second_2fprotos_2fpreprocess_2eproto_getter, &descriptor_table_second_2fprotos_2fpreprocess_2eproto_once,
      file_level_metadata_second_2fprotos_2fpreprocess_2eproto[6]);
}

// ===================================================================

class RandomGlobalRotation::_Internal {
 public:
};

RandomGlobalRotation::RandomGlobalRotation(::PROTOBUF_NAMESPACE_ID::Arena* arena,
                         bool is_message_owned)
  : ::PROTOBUF_NAMESPACE_ID::Message(arena, is_message_owned) {
  SharedCtor(arena, is_message_owned);
  // @@protoc_insertion_point(arena_constructor:second.protos.RandomGlobalRotation)
}
RandomGlobalRotation::RandomGlobalRotation(const RandomGlobalRotation& from)
  : ::PROTOBUF_NAMESPACE_ID::Message() {
  RandomGlobalRotation* const _this = this; (void)_this;
  new (&_impl_) Impl_{
      decltype(_impl_.min_rad_){}
    , decltype(_impl_.max_rad_){}
    , /*decltype(_impl_._cached_size_)*/{}};

  _internal_metadata_.MergeFrom<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(from._internal_metadata_);
  ::memcpy(&_impl_.min_rad_, &from._impl_.min_rad_,
    static_cast<size_t>(reinterpret_cast<char*>(&_impl_.max_rad_) -
    reinterpret_cast<char*>(&_impl_.min_rad_)) + sizeof(_impl_.max_rad_));
  // @@protoc_insertion_point(copy_constructor:second.protos.RandomGlobalRotation)
}

inline void RandomGlobalRotation::SharedCtor(
    ::_pb::Arena* arena, bool is_message_owned) {
  (void)arena;
  (void)is_message_owned;
  new (&_impl_) Impl_{
      decltype(_impl_.min_rad_){0}
    , decltype(_impl_.max_rad_){0}
    , /*decltype(_impl_._cached_size_)*/{}
  };
}

RandomGlobalRotation::~RandomGlobalRotation() {
  // @@protoc_insertion_point(destructor:second.protos.RandomGlobalRotation)
  if (auto *arena = _internal_metadata_.DeleteReturnArena<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>()) {
  (void)arena;
    return;
  }
  SharedDtor();
}

inline void RandomGlobalRotation::SharedDtor() {
  GOOGLE_DCHECK(GetArenaForAllocation() == nullptr);
}

void RandomGlobalRotation::SetCachedSize(int size) const {
  _impl_._cached_size_.Set(size);
}

void RandomGlobalRotation::Clear() {
// @@protoc_insertion_point(message_clear_start:second.protos.RandomGlobalRotation)
  uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  ::memset(&_impl_.min_rad_, 0, static_cast<size_t>(
      reinterpret_cast<char*>(&_impl_.max_rad_) -
      reinterpret_cast<char*>(&_impl_.min_rad_)) + sizeof(_impl_.max_rad_));
  _internal_metadata_.Clear<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>();
}

const char* RandomGlobalRotation::_InternalParse(const char* ptr, ::_pbi::ParseContext* ctx) {
#define CHK_(x) if (PROTOBUF_PREDICT_FALSE(!(x))) goto failure
  while (!ctx->Done(&ptr)) {
    uint32_t tag;
    ptr = ::_pbi::ReadTag(ptr, &tag);
    switch (tag >> 3) {
      // float min_rad = 1;
      case 1:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 13)) {
          _impl_.min_rad_ = ::PROTOBUF_NAMESPACE_ID::internal::UnalignedLoad<float>(ptr);
          ptr += sizeof(float);
        } else
          goto handle_unusual;
        continue;
      // float max_rad = 2;
      case 2:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 21)) {
          _impl_.max_rad_ = ::PROTOBUF_NAMESPACE_ID::internal::UnalignedLoad<float>(ptr);
          ptr += sizeof(float);
        } else
          goto handle_unusual;
        continue;
      default:
        goto handle_unusual;
    }  // switch
  handle_unusual:
    if ((tag == 0) || ((tag & 7) == 4)) {
      CHK_(ptr);
      ctx->SetLastTag(tag);
      goto message_done;
    }
    ptr = UnknownFieldParse(
        tag,
        _internal_metadata_.mutable_unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(),
        ptr, ctx);
    CHK_(ptr != nullptr);
  }  // while
message_done:
  return ptr;
failure:
  ptr = nullptr;
  goto message_done;
#undef CHK_
}

uint8_t* RandomGlobalRotation::_InternalSerialize(
    uint8_t* target, ::PROTOBUF_NAMESPACE_ID::io::EpsCopyOutputStream* stream) const {
  // @@protoc_insertion_point(serialize_to_array_start:second.protos.RandomGlobalRotation)
  uint32_t cached_has_bits = 0;
  (void) cached_has_bits;

  // float min_rad = 1;
  static_assert(sizeof(uint32_t) == sizeof(float), "Code assumes uint32_t and float are the same size.");
  float tmp_min_rad = this->_internal_min_rad();
  uint32_t raw_min_rad;
  memcpy(&raw_min_rad, &tmp_min_rad, sizeof(tmp_min_rad));
  if (raw_min_rad != 0) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteFloatToArray(1, this->_internal_min_rad(), target);
  }

  // float max_rad = 2;
  static_assert(sizeof(uint32_t) == sizeof(float), "Code assumes uint32_t and float are the same size.");
  float tmp_max_rad = this->_internal_max_rad();
  uint32_t raw_max_rad;
  memcpy(&raw_max_rad, &tmp_max_rad, sizeof(tmp_max_rad));
  if (raw_max_rad != 0) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteFloatToArray(2, this->_internal_max_rad(), target);
  }

  if (PROTOBUF_PREDICT_FALSE(_internal_metadata_.have_unknown_fields())) {
    target = ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
        _internal_metadata_.unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(::PROTOBUF_NAMESPACE_ID::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:second.protos.RandomGlobalRotation)
  return target;
}

size_t RandomGlobalRotation::ByteSizeLong() const {
// @@protoc_insertion_point(message_byte_size_start:second.protos.RandomGlobalRotation)
  size_t total_size = 0;

  uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  // float min_rad = 1;
  static_assert(sizeof(uint32_t) == sizeof(float), "Code assumes uint32_t and float are the same size.");
  float tmp_min_rad = this->_internal_min_rad();
  uint32_t raw_min_rad;
  memcpy(&raw_min_rad, &tmp_min_rad, sizeof(tmp_min_rad));
  if (raw_min_rad != 0) {
    total_size += 1 + 4;
  }

  // float max_rad = 2;
  static_assert(sizeof(uint32_t) == sizeof(float), "Code assumes uint32_t and float are the same size.");
  float tmp_max_rad = this->_internal_max_rad();
  uint32_t raw_max_rad;
  memcpy(&raw_max_rad, &tmp_max_rad, sizeof(tmp_max_rad));
  if (raw_max_rad != 0) {
    total_size += 1 + 4;
  }

  return MaybeComputeUnknownFieldsSize(total_size, &_impl_._cached_size_);
}

const ::PROTOBUF_NAMESPACE_ID::Message::ClassData RandomGlobalRotation::_class_data_ = {
    ::PROTOBUF_NAMESPACE_ID::Message::CopyWithSourceCheck,
    RandomGlobalRotation::MergeImpl
};
const ::PROTOBUF_NAMESPACE_ID::Message::ClassData*RandomGlobalRotation::GetClassData() const { return &_class_data_; }


void RandomGlobalRotation::MergeImpl(::PROTOBUF_NAMESPACE_ID::Message& to_msg, const ::PROTOBUF_NAMESPACE_ID::Message& from_msg) {
  auto* const _this = static_cast<RandomGlobalRotation*>(&to_msg);
  auto& from = static_cast<const RandomGlobalRotation&>(from_msg);
  // @@protoc_insertion_point(class_specific_merge_from_start:second.protos.RandomGlobalRotation)
  GOOGLE_DCHECK_NE(&from, _this);
  uint32_t cached_has_bits = 0;
  (void) cached_has_bits;

  static_assert(sizeof(uint32_t) == sizeof(float), "Code assumes uint32_t and float are the same size.");
  float tmp_min_rad = from._internal_min_rad();
  uint32_t raw_min_rad;
  memcpy(&raw_min_rad, &tmp_min_rad, sizeof(tmp_min_rad));
  if (raw_min_rad != 0) {
    _this->_internal_set_min_rad(from._internal_min_rad());
  }
  static_assert(sizeof(uint32_t) == sizeof(float), "Code assumes uint32_t and float are the same size.");
  float tmp_max_rad = from._internal_max_rad();
  uint32_t raw_max_rad;
  memcpy(&raw_max_rad, &tmp_max_rad, sizeof(tmp_max_rad));
  if (raw_max_rad != 0) {
    _this->_internal_set_max_rad(from._internal_max_rad());
  }
  _this->_internal_metadata_.MergeFrom<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(from._internal_metadata_);
}

void RandomGlobalRotation::CopyFrom(const RandomGlobalRotation& from) {
// @@protoc_insertion_point(class_specific_copy_from_start:second.protos.RandomGlobalRotation)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}

bool RandomGlobalRotation::IsInitialized() const {
  return true;
}

void RandomGlobalRotation::InternalSwap(RandomGlobalRotation* other) {
  using std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  ::PROTOBUF_NAMESPACE_ID::internal::memswap<
      PROTOBUF_FIELD_OFFSET(RandomGlobalRotation, _impl_.max_rad_)
      + sizeof(RandomGlobalRotation::_impl_.max_rad_)
      - PROTOBUF_FIELD_OFFSET(RandomGlobalRotation, _impl_.min_rad_)>(
          reinterpret_cast<char*>(&_impl_.min_rad_),
          reinterpret_cast<char*>(&other->_impl_.min_rad_));
}

::PROTOBUF_NAMESPACE_ID::Metadata RandomGlobalRotation::GetMetadata() const {
  return ::_pbi::AssignDescriptors(
      &descriptor_table_second_2fprotos_2fpreprocess_2eproto_getter, &descriptor_table_second_2fprotos_2fpreprocess_2eproto_once,
      file_level_metadata_second_2fprotos_2fpreprocess_2eproto[7]);
}

// @@protoc_insertion_point(namespace_scope)
}  // namespace protos
}  // namespace second
PROTOBUF_NAMESPACE_OPEN
template<> PROTOBUF_NOINLINE ::second::protos::Preprocess*
Arena::CreateMaybeMessage< ::second::protos::Preprocess >(Arena* arena) {
  return Arena::CreateMessageInternal< ::second::protos::Preprocess >(arena);
}
template<> PROTOBUF_NOINLINE ::second::protos::DatabasePreprocessingStep*
Arena::CreateMaybeMessage< ::second::protos::DatabasePreprocessingStep >(Arena* arena) {
  return Arena::CreateMessageInternal< ::second::protos::DatabasePreprocessingStep >(arena);
}
template<> PROTOBUF_NOINLINE ::second::protos::DBFilterByDifficulty*
Arena::CreateMaybeMessage< ::second::protos::DBFilterByDifficulty >(Arena* arena) {
  return Arena::CreateMessageInternal< ::second::protos::DBFilterByDifficulty >(arena);
}
template<> PROTOBUF_NOINLINE ::second::protos::DBFilterByMinNumPointInGroundTruth_MinNumPointPairsEntry_DoNotUse*
Arena::CreateMaybeMessage< ::second::protos::DBFilterByMinNumPointInGroundTruth_MinNumPointPairsEntry_DoNotUse >(Arena* arena) {
  return Arena::CreateMessageInternal< ::second::protos::DBFilterByMinNumPointInGroundTruth_MinNumPointPairsEntry_DoNotUse >(arena);
}
template<> PROTOBUF_NOINLINE ::second::protos::DBFilterByMinNumPointInGroundTruth*
Arena::CreateMaybeMessage< ::second::protos::DBFilterByMinNumPointInGroundTruth >(Arena* arena) {
  return Arena::CreateMessageInternal< ::second::protos::DBFilterByMinNumPointInGroundTruth >(arena);
}
template<> PROTOBUF_NOINLINE ::second::protos::PreprocessingStep*
Arena::CreateMaybeMessage< ::second::protos::PreprocessingStep >(Arena* arena) {
  return Arena::CreateMessageInternal< ::second::protos::PreprocessingStep >(arena);
}
template<> PROTOBUF_NOINLINE ::second::protos::RandomGlobalScaling*
Arena::CreateMaybeMessage< ::second::protos::RandomGlobalScaling >(Arena* arena) {
  return Arena::CreateMessageInternal< ::second::protos::RandomGlobalScaling >(arena);
}
template<> PROTOBUF_NOINLINE ::second::protos::RandomGlobalRotation*
Arena::CreateMaybeMessage< ::second::protos::RandomGlobalRotation >(Arena* arena) {
  return Arena::CreateMessageInternal< ::second::protos::RandomGlobalRotation >(arena);
}
PROTOBUF_NAMESPACE_CLOSE

// @@protoc_insertion_point(global_scope)
#include <google/protobuf/port_undef.inc>
